procedure lincor (index) 
#
# Do linearity correction for an icp image and create an ilp image
#
#   Paul Martini
#   2003 June 10
#

int index      {prompt = "File number: "}
bool overwrite {yes, prompt = "Overwrite if output already exists?" }
bool verbose   {no, prompt = "Print verbose output?"} 
real linthresh {8000., prompt = "Linearity Threshold"}
real c1	       {4.291443e+01, prompt = "Coefficient 1"} 
real c2	       {9.752524e-01, prompt = "Coefficient 2"} 
real c3	       {1.962545e-06, prompt = "Coefficient 3"} 

begin

  real c1loop, c2loop, c3loop, loopthresh, ncombine
  string onumber, image, output 

  padindex(index, ngoal=5)
  onumber = padindex.output

  image = "icp_"//onumber//".fits" 
  output = "ilp_"//onumber//".fits"

# Check that the input is present and the output not

  if (!access(image)) { 
    print(" LINCOR: Error, image ",image," not found")
    clbye()
  }

  if (access(output)) { 
    if (overwrite) {
      delete(output, ver-)
    } else { 
      print(" LINCOR: file ",output," already exists")
      print(" LINCOR: run with overwrite=yes to recalculate")
      clbye()
    }
  }

# Calculate the coefficients based on the number of loops
 
  imgets(image,"NCOMBINE",>&"dev$null")
  ncombine = real(imgets.value)

  c1loop = c1*ncombine
  c2loop = c2
  c3loop = c3/ncombine
  loopthresh = linthresh*ncombine

  imcalc(image, output, equals="if im1 .le. "//loopthresh//" then im1 else "//c1loop//" + "//c2loop//"*im1 +  "//c3loop//"*sqr(im1)", pixtype="old", verbose=verbose)

end
