procedure p_loopavg (number)
#
# Given a frame number average all images of the form
# izp_number_001.fits, etc. Output image is icp_number.fits
# and is the pclipped average of the inputs.
#
#   Paul Martini
#   2003 Apr 12
#
# Modifications: 
#   2003 May 2:  Avg with pclip, rather than straight addition, if nloops>=3. 
#                EXPTIME has summed EXPTIME, OEXPTIME is orig value per loop.
#   2003 June 8: Added argument for raw data path set in parspanic.cl
#
# Modifications by SEP:
#   2003 Aug 12: Removed all the parspanic stuff. 
#                Changed input pix to linearized 'izp' versions.
#

int number     {prompt = "First image number" }
bool overwrite {no, prompt = "Overwrite if output already exists?"} 
string output  {"",prompt = "Return value of output image name"}

begin

  int i, nloops
  real oldexp, newexp
  string out, onumber

  if (access("loopsumtemp")) { delete ("loopsumtemp", ver-) }

# Force the number to have five digits, pad front with zeros if nec.
    
  padindex(number) 
  onumber = padindex.output

# Get the number of loops:

  imgets("izp_"//onumber//"_001.fits","NLOOPS",>&"dev$null")
  nloops=real(imgets.value)

# Make the file to combine images 

  print ("izp_"//onumber//"_001.fits", > "loopsumtemp")
  for (i=2; i<=nloops; i+=1) {
    if (i < 10) { print ("izp_"//onumber//"_00"//i//".fits", >> "loopsumtemp") }
    else if (i < 100) { print ("izp_"//onumber//"_0"//i//".fits", >> "loopsumtemp") }
    else { print ("izp_"//onumber//"_"//i//".fits", >> "loopsumtemp") }
  }

  out = "icp_"//onumber//".fits"

  if (overwrite) { if (access(out) ) { imdelete (out, ver-) } } 
  
  if (access(out)) {
    print(" LOOPSUM: file ",out," already exists")
    print(" LOOPSUM: run with overwrite=yes to recalculate") 
    bye 
  } else { 
    if (nloops < 3) { 
      imcombine("@loopsumtemp", output=out, rejmask="", plfile="", sigma="", combine="average",reject="none",offsets="none", masktyp="badval",project=no,pclip=-0.5,scale="none")
    } else {
      imcombine("@loopsumtemp", output=out, rejmask="", plfile="", sigma="", combine="average",reject="pclip",offsets="none", masktyp="badval",project=no,pclip=-0.5,scale="none")
    }
    imgets(out,"EXPTIME",>&"dev$null")
    oldexp=real(imgets.value)
    newexp=nloops*oldexp
    hedit (out, fields="OEXPTIME", value=oldexp, update=yes, verify=no, show=no, add=yes)
    hedit (out, fields="EXPTIME", value=newexp, update=yes, verify=no, show=no, add=no)
    output = out
  }

# Clean up

  delete ("loopsumtemp", ver-)

end

