procedure p_make_twi_flat ( camera, f1, f2 )

string camera
int f1, f2
struct *list1

#	Assumes that 'icp' linearized and loop-combined frames 
#	exist. 'f1' and 'f2' are the starting and ending 
#	frame numbers (of valid non-saturated) frames of a set  
#	taken as the sky is changing during twilight.
#
#	It is assumed that 'chk_headers' and 'fix_headers' have 
#	been run and that therefore it is not necessary to check  
#	for filter consistency.
#
#	The existence of frames within a set is not assumed (in
#	case it has been necessary to delete some) but is worked
#	around. However frames f1 and f2 must exist.
#
#	This script assumes that the data have been taken
#	according to the procedure specified for twiflats
#	in the PANIC camera control gui. IN PARTICULAR, the
#	telescope must be moved between frames so that field
#	stars can be medianed away. 
#
#	If there are any bright stars in these frames, this 
#	script will not work well unless N > 20 or even 25.
#
#	Assume there are N valid frames. The frames are combined
#	according to the following prescription:
#	take the first and last int(N/3) frames and form
#	differences. For example, suppose 20 frames total were 
#	taken as the sky was fading. Then the pairs of frames are
#
#	   bright(i) - faint(j)
#                   
#	     1          15 
#	     2          16
#	     3          17
#	     4          18
#	     5          19
#	     6          20
#
#	It is assumed FOR NOW that they all have the same exposure time.
#	
#	Giving six frames. Normalize them and imcombine as median. 
#	
#	dvorak:/home/persson/iraf/panic_scripts/p_make_twi_flat.cl
#	Rev: 28nov2003

begin
	int i, nf, p3
	string onumber1, onumber2, date, onumber
	string im_i, im_j, im_d, im_d_n

	string filter1_f1, filter2_f1, filter1_f2, filter2_f2

#	Get filters and date out of headers

	padindex(f1) ; onumber1 = padindex.output
	padindex(f2) ; onumber2 = padindex.output

	keypar("icp_"//onumber1//".fits", "FILTER1",silent+)
	filter1_f1 = keypar.value

	print ( filter1_f1 )

	keypar("icp_"//onumber1//".fits", "FILTER2",silent+)
	filter2_f1 = keypar.value

	print ( filter2_f1 )

	keypar("icp_"//onumber2//".fits", "FILTER1",silent+)
	filter1_f2 = keypar.value

	keypar("icp_"//onumber2//".fits", "FILTER2",silent+)
	filter2_f2 = keypar.value

	keypar("icp_"//onumber1//".fits", "NIGHT",silent+)
	date = "_"//keypar.value

	if ( filter1_f1 == filter1_f2 && filter2_f1 == filter2_f2)	{

#	Make a list of the frames that exist

	pclear ( "twi_list_1" ) ; pclear ( "twi_list_2" )
	pclear ( "twi_list" )   ; pclear ( "twi_list_rev" )
	pclear ( "diff_list" )

	nf = 0
	for ( i = f1 ; i <= f2 ; i += 1 ) 	{

	  padindex(i) ; onumber = padindex.output
	  if ( access ( "icp_"//onumber//".fits" )) 	{
	    nf += 1
            print ( "icp_"//onumber//".fits", >> "twi_list" )
		}	else	{
					}
									}
!sort -r "twi_list" > "twi_list_rev"

	p3 = int (( nf )/3. )

#	p3 is approximately 1/3 of 'nf', the total number of 
#	frames that actually exist, between f1 and f2 inclusive.
#	
#	Now make a list of the p3 (bright) frames that exist:

	list1 = "" ; list1 = "twi_list" ; nf =0
	while ( fscan ( list1, im_i) != EOF ) 	{
	  nf += 1
	  if ( nf <= p3 )	{
	    print ( im_i, >> "twi_list_1" )
					}	else	{}
										}

#	Now make a list of the p3 (faint) frames that exist:

	list1 = "" ; list1 = "twi_list_rev" ; nf =0
	while ( fscan ( list1, im_j) != EOF ) 	{
	  nf += 1
	  if ( nf <= p3 )	{
	    print ( im_j, >> "twi_list_2" )
					}	else	{}
										}

! sort -r twi_list_2 | paste twi_list_1 - > diff_list

#	Form difference frames 

	list1 = "" ; list1 = "diff_list"
	while ( fscan ( list1, im_i, im_j) != EOF ) {

	  im_d   = "icp_"//i//"_"//j//".fits"
	  im_d_n = "icp_"//i//"_"//j//"_n"//".fits"

	  pclear ( im_d ) ; pclear ( im_d_n )
	  imarith ( im_i, "-", im_j, im_d )
	  p_normalize ( im_d, im_d_n )
          print ( im_d_n, >> "diff_list" )
											}

#	Combine normalized differences 

	pclear ("nptflat"//filter1_f1//filter2_f1//date)

	imcombine ( "@"//"diff_list", "nptflat"//filter1_f1//filter2_f1//date, combine="median",scale="none",zero="median",reject="sigclip")
	p_store ( "nptflat"//filter1_f1//filter2_f1//date )

						}	else	{
	print ( "" )
	print ( "  >> SORRY: Filters are not consistent " )
	print ( "" )
									}		

end

	  















	  













