procedure pr_1go ( n1, n2, flag )

int n1, n2
string flag
struct *list1

#  PANIC pipeline for galaxies, standard stars, and 
#  crowded field data.
#  For galaxy data typically square-9 dithers were used.
#  For standard stars typically dice-5 dithers were used. 
#
#  Decode 'flag' to process:
#
#  n      routines that apply to all frames in the night, n1 - n2
#  u      uncrowded frames, e.g., LCIRS galaxies - square-9 dithers
#  c      crowded frames, e.g., rich starfields with separate skies 
#  s      standard stars - dice-5 dithers
#
#  If any of these flags are set, it is expected that the corresponding 
#  pairs files exist:
#
#   u    'unc_repairs'  should exist
#   s    'std_repairs'    "      " 
#
#  In this revised version for repairing data, crowded frames 
#  will not be an option.  
#
#
#

begin
	int nf, i, p1, p2, p3, p4
	bool naf, unc, crd, std
	string onumber, date

	naf = no ; unc = no ; crd = no ; std = no
	nf = strlen(flag)
	  for ( i = 1; i <= nf; i += 1 ) 	{
            if ( substr(flag,i,i) == "n") naf = yes 
            if ( substr(flag,i,i) == "u") unc = yes 
            if ( substr(flag,i,i) == "c") crd = yes 
            if ( substr(flag,i,i) == "s") std = yes
						}

#  These are the procedures that apply to all frames
#  Not used in this repair version

#	if(naf) {
#
#	  p_linearize ( "p", n1, n2 )
#	  p_defaults ( n1 )
#	  p_loops_combine  ( "izp", n1, n2 )            
#	  p_dark ( n1, n2 )
#	  p_subdk ( n1, n2 )
#
#	if ( access ( "dflat_pairs" ))	{
#	list1 = "" ; list1 = "dflat_pairs"
#	while ( fscan ( list1, p1, p2 ) != EOF ) {
#	  p_dflat ( p1, p2 )
#						 }
#					} else {}
#
#	if ( access ( "tflat_pairs" ))	{
#	list1 = "" ; list1 = "tflat_pairs"
#	while ( fscan ( list1, p1, p2 ) != EOF ) {
#	  p_tflat ( p1, p2 )
#						 }
#					} else {}
#
#	  p_flatten ( n1, n2 )
#
#	}  else  { }
#
#  Done with procedures that apply to all frames in the night or run.
#  Dealing with 'idf' frames from now on. 
#
#  Now come the several variants of the stacking procedures
#  Do uncrowded frame (e.g. faint galaxy) stacks

	if(unc)	{
	  if ( access ( "unc_repairs" ))	{

	list1 = "" ; list1 = "unc_repairs"
	while ( fscan ( list1, p1, p2 ) != EOF )   {

	  p_subdk ( p1, p2 )
	  p_flatten ( p1, p2 )
	  p_sky1   ( "idf", p1, p2 ) 
	  p_dither ( p1, p2 )
	  p_stk1   ( p1, p2 )
	  p_objmask( p1, p2 )
	  pr_skysub ( p1, p2 ) 
	  p_stack  ( p1, p2, "test_"//p1//"_"//p2 ) 
	  p_hchen  ( "comblist_test_"//p1//"_"//p2, "chen_"//p1//"_"//p2 )
	
	disp ( "chen_"//p1//"_"//p2, 1 )
						   }
					} else {}
		} else {}
#
#  Do crowded frame stacks; off-source sky frames were done before and/or after
 
	if(crd)	{
	  if ( access ( "crd_pairs" ))	{

#  First go through all of-source "sky" frames, creating stacks and cleaned skies,
#  for each set of up to two off-source dither locations. The 'crd_pairs' file is 
#  first processed to make a single non-repeating list of pairs of off-source
#  position frames; this is 'crdlist'.

	mk_crdlist ("crd_pairs")

#  Now call the usual stacking routines, saving the sky frames for 
#  all off-source pairs. In the following set of calls p1,p2 are 
#  the off-source frames, originally p3,p4 and p5,p6 in 'crd_pairs'.

	list1 = "" ; list1 = "crdlist"
	while ( fscan ( list1, p1, p2 ) != EOF )   {

	    p_sky1   ( "idf", p1, p2 ) 
	    p_dither ( p1, p2 )
	    p_stk1   ( p1, p2 )
	    p_objmask( p1, p2 )
	    p_skysub ( p1, p2 ) 
						   }
#
#  Now call a special sky subtraction routine that expects the two 
#  off-source sky frames 'sky_000p3' and 'sky_000p5' to exist. They
#  do, because 'p_sky1' was just run on all the off-source pairs.

#	NEED WORK HERE  'p_skysub' has also created 'sky_000p3_000p4'
#	as well as 'sky_000p5_000p6'. So could use either one in 'p_sky3'.
#	In general the 'sky_000p3_000p4'-type should be better because 
#	the stars have been masked out.  

	list1 = "" ; list1 = "crd_pairs"
	while ( fscan ( list1, p1, p2, p3, p4, p5, p6 ) != EOF )   {

	    p_sky3 ( p1, p2, p3, p4, p5, p6 ) 
	    p_dither ( p1, p2 )
	    p_stk1 ( p1, p2, "test_"//p1//"_"//p2 ) 
            p_hchen ( "comblist_test_"//p1//"_"//p2, "chen_"//p1//"_"//p2 )

								   }
					} else {}
		} else {}


#	Do stacks of all standard stars.

	if (std) 	{

	if ( access ( "std_repairs" ))	{

#	pclear ( "ngt_"//date )
	list1 = "" ; list1 = "std_repairs"
	while ( fscan ( list1, p1, p2, p3, p4 ) != EOF ) {

	  p_subdk ( p1, p2 )
	  p_flatten ( p1, p2 )
	  p_sky2 ( p1, p2, p3, p4 ) 
	  p_dither ( p1, p2 )
	  p_stk1 ( p1, p2 )
	  p_std ( p1, p2 )
					 }

#	  p_sort ( "ngt_"//date )	
#         padindex (n1) ; onumber=padindex.output
#	  p_get_date ( "irp", n1 ) ; date = get_date.output

					
					} else {}

	}  else  { }

end
