procedure p_dflat ( p1, p2 )

int p1, p2
struct *list1

#  /home/persson/iraf/panic_scripts/p_dflat.cl
#
#  Produces normalized domeflats for the filter and date 
#  in the frame header. 
#  Assumes that 'icd' linearized, loop-combined, and 
#  dark-subtracted frames exist. 'p1' and 'p2' are the 
#  lamp-on and lamp-off frame numbers. 
#
#  Also computes the badpixel mask which will be the last 
#  one processed in the list of 'dflat_pairs'. 

#  The filter is taken out of each header and checked for 
#  consistency.
#
#  05Aug2004: This routine now only used to make badpixel
#             masks. 

begin
   string onumber1, onumber2, date, filter_p1, filter_p2, default_path
   real frmed, frmed7
   string ldark, exposure

#   Get filter and date out of headers

   padindex(p1) ; onumber1 = padindex.output
   padindex(p2) ; onumber2 = padindex.output

   keypar("icd_"//onumber1//".fits", "FILTER",silent+)
   filter_p1 = str(keypar.value)

   keypar("icd_"//onumber2//".fits", "FILTER",silent+)
   filter_p2 = str(keypar.value)

   keypar("icd_"//onumber1//".fits", "NIGHT",silent+)
   date = str(keypar.value)

#   Check for filter consistency

   if ( filter_p1 == filter_p2)   {

   pclear ("flat") ; pclear ( "mask" )
   imarith ( "icd_"//onumber1, "-", "icd_"//onumber2, "flat" )

#   Make sure it is a positive image now

   p_store ("flat")
   keypar("flat","FRMEDIAN",silent+)
   frmed = real(keypar.value)

   pclear ( "hot1" ) ; pclear ( "hot2")
   if ( frmed <= 0. ) {
      imarith ( "flat", "*", -1., "flat" )
      imcopy ( "icp_"//onumber1, "hot1" )
   }
   if ( frmed > 0. ) {
      imcopy ( "icp_"//onumber2, "hot1" )
   }

#  ID hot and negative pixels in the lamps-off frame

   pclear ( "fr_stats" ) ; pclear ( "fr_stats_file" )
   imstat ( "hot1"//"[41:984,41:984]", fields="midpt", lower=0, upper=1000, > "fr_stats")
   ! /usr/bin/tail -1 fr_stats > fr_stats_file

   list1 = "" ; list1 = "fr_stats_file"
   while ( fscan ( list1, frmed ) != EOF ) {}
   frmed7 = frmed * 11.
   
   imcopy ( "hot1", "hot2" )
   imreplace ( "hot1", value=30000, lower=frmed7, upper=INDEF )
   frmed7 = -frmed7
   imreplace ( "hot1", value=30000, lower=INDEF, upper=frmed7 )
   imarith ( "hot1", "-", "hot2", "hot1" )
   imarith ( "hot1", "/", "hot1", "hot1", divzero=0. )

#  ID hot pixels in the 120-sec dark - these may be flickering
#  ID NEGATIVE pixels in the 120-sec dark - these flickered and turned off between reads
#  Hold on there!  What if there's no 120-sec dark?  If there isn't, we
#  should just look for the longest one.
   if ( access ("dk_120_"//date) ) {
      ldark = "dk_120_"//date
   } else {
      # find the longest exposure time:
      ! ls dk* | cut -f 2 -d "_" | sort -n | /usr/bin/tail -1 > dark_exp
      list1 = "" ; list1 = "dark_exp"
      while ( fscan ( list1, exposure) != EOF ) {}
      ldark = "dk_"//exposure//"_"//date
   }
   pclear ( "hot2")
   imcopy ( ldark , "hot2")
   imreplace ( "hot2", value=30000, lower=100., upper=INDEF )
   imreplace ( "hot2", value=30000, lower=INDEF, upper=-100. )
   imarith ( "hot2", "-", ldark, "hot2" )
   imarith ( "hot2", "/", "hot2", "hot2", divzero=0. )

#  Now hot and negative pixels have value 1.

#    Normalize to the mode of the lamp-on minus lamp-off difference image
#   Note that the output image is positive.
#   Fix edges, which also stores image statistics.

   p_norm ( "flat", "flat", "median" )
   imcopy ( "flat", "mask" )

#   Could do an imsurfit here to further tighten badpixel definition.  
#   Maybe just remove overall tilt?


#   Get rid of dead and hot pixels. A PIXEL CAN
#   HAVE A QE NO LESS THAN 66% or 150% RELATIVE TO THE MEDIAN OF ALL.
#
#   Could do an imsurfit here to further tighten badpixel definition.  

   imreplace ( "flat", value=1, lower=INDEF, upper=0.60 )
   imreplace ( "flat", value=1, lower=1.5, upper=INDEF )

#   Now create badpixel mask using the same definitions of 'dead' and 'hot'
#   pixels. First flatten the 'mask'.

   imarith ( "mask", "/", "flat", "mask", divzero=0. )

#  Here are the badpixels to mask out:

   imreplace ( "mask", value=0, lower=INDEF, upper=0.60 )
   imreplace ( "mask", value=0, lower=1.5, upper=INDEF )
   imreplace ( "mask[255:261,864:870]", 0 )
   imreplace ( "mask[289:291,128:130]", 0 )
   imreplace ( "mask[288:288,130:130]", 0 )
   imreplace ( "mask[289:292,127:131]", value=0 )
   imreplace ( "mask[733:736,165:168]", value=0 )
   imreplace ( "mask[193:193,925:925]", value=0 )

#  Here heavily mask the edges:
 
   imreplace ( "mask[1:1024,985:1024]", value=0 )
   imreplace ( "mask[1:1024,1:38]", value=0 )
   imreplace ( "mask[1:43,1:1024]", value=0 )
   imreplace ( "mask[960:1024,1:1024]", value=0 )

   imarith ( 1., "-", "mask", "mask" )

#  Now bad pixels have value 1; add in hot pixels

   imarith ( "mask", "+", "hot1", "mask" )
   imarith ( "mask", "+", "hot2", "mask" )
   imreplace ( "mask", value=1, lower=1.01, upper=INDEF )

   pclear ( "mask_"//date//".pl" )
   imcopy ( "mask.fits", "mask_"//date//".pl" )
   display ( "mask_"//date//".pl", 1 )

#   default_path = "home$/panic_files/bp_masks/"
#   pclear ( default_path//"mask_"//date//".pl")
#   copy ( "mask_"//date//".pl", default_path )
  
                     }   else   {
   print ( "" )
   print ( "  >> SORRY: Filters are not consistent " )
   print ( "" )
                     }
end





