procedure p_dither (pre, p1, p2)

##############################################################
# Written by P McCarthy and J Wilson, 8/18/99		     #
#							     #
# For each dither sequence, finds offset of each frame w/r/t #
# the REFERENC frame by comparing frame's coordinates for a  #
# user selected group of stars to the coordinates for those  #
# stars in the REFERENC frame. Writes offset values (in      #
# pixels) into the header of the isx and icx files as XOFF   #
# and YOFF, as well as the REFERENC frame (keyword REFERENC).#
# Input is a list of the isx*.fits files.		     #
# NOTE: Must start display tool before running this task     #
#							     #
##############################################################

string pre
int p1, p2
struct	  *list1, *list2

begin

string	  filename, zptfile, ycoor, xcoor, rapix, decpix
string    idfname, prename, rootname, anothername, type, frame
string    astro_list, root, onumber, filelist
int	  i, xref, yref
file	  init.dat, xycoor.dat, newinits.dat
file      xycoor1.dat, xycoor2.dat, xycoor3.dat, xycoor4.dat
real	  x, y, x1, y1, x2, y2, x3, y3, xrasec, ydecsec
real      newxinit, newyinit, xinit, yinit, ra, dec, rasec, decsec
real      xstart, ystart, first_ra, first_dec, first_asec, first_dsec
real      xsum, ysum, k, rastart, decstart, raref, decref
real 	  rotang, sense, xfid, yfid, psf_e, psf_d

   #	09mar2004 SEP: convert input to p1,p2 format

	p_pre_list ( pre, p1, p2 )	
	filelist = pre//"_list"

# initialize variables
   i = 0		#counting variable
   ra = 0.0		#asecs value
   dec = 0.0		#dsecs value
   xstart = 0.0		#
   ystart = 0.0		#
   first_ra = 0.0	#
   first_dec = 0.0	# 
   first_asec = 0.0
   first_dsec = 0.0
pclear ("xycoor.dat")
pclear ("xycoor1.dat")
pclear ("xycoor2.dat")
pclear ("xycoor3.dat")
pclear ("xycoor4.dat")
pclear ("results.dat")
pclear ("xy.dat")
pclear ("log.dat")
pclear ("newinits.dat")
pclear ("init.dat")
pclear ("slist")

# loop through files calculating offset and new approx co-ors for fiducial;
  list1 = ""; list1=filelist
  while (fscan (list1,filename) != EOF)  {

# find the first position in each dither sequence 

       keypar (filename, "asecs", silent+)
       xstart = real(keypar.value)
       keypar (filename, "dsecs", silent+)
       ystart = real(keypar.value)
       keypar (filename, "ra", silent+)
       rastart = real(keypar.value)
       keypar (filename, "dec", silent+)
       decstart = real(keypar.value)

       # determine scale of coor change to see if dither or raster
       xref = int(xstart - first_asec)	#ref frame coors - this frame coors
       yref = int(ystart - first_dsec)	#ref frame coors - this frame coors
       raref = real(rastart - first_ra)
       decref = real(decstart - first_dec)

       #determine if pointing has changed w/r/t previous frame
       if ( xref >=  108)  i = 0  
       if ( xref <=  -108) i = 0  
       if ( yref >=  108)  i = 0
       if ( yref <=  -108) i = 0
      
       if ( raref >=  0.3400000)  i = 0
       if ( raref <=  -0.3400000) i = 0
       if ( decref >= 0.3400000)  i = 0
       if ( decref <= -0.3400000) i = 0

       i = i + 1

 
       if (i == 1) 
	   {
	   #Populate REFERENC file variables

	    zptfile=filename
print ("zptfile=reffile="//zptfile)
	    first_asec = xstart
	    first_dsec = ystart
            first_ra = rastart 
            first_dec = decstart 


           pclear ("init.dat")
           pclear ("slist")

           rootname = substr(filename,1,9)
           idfname = "idf_"//substr(filename,5,9)

#	Query ROTANG parameter in header of first frame to get rotation angle
#	of IMB. rotang = 46.5 means normal (unreversed), 226.5 means reversed.

	  keypar(idfname,"ROTANG",silent+)
	  rotang = real(keypar.value)

	if ( rotang <= 50. ) sense = -1.
	if ( rotang >= 200. ) sense = 1. 

#	If fiducial star has already been found using 'p_quick_stack'
#	for example, then this interactive step can be skipped.

	xfid = 0. ; yfid = 0.

	  keypar(idfname,"XFID",silent+)
	  if(keypar.found) xfid = real(keypar.value)

	  keypar(idfname,"YFID",silent+)
	  if(keypar.found) yfid = real(keypar.value)

	if ( xfid >= 50. && yfid >= 50. )	{

	  print ( xfid,"  ",yfid, > "init.dat" )
								    }  else {

           display (rootname, 1, erase-, bor+, fill+, zs-,zr-,z1=-200,z2=200, xmag=0.5, ymag=0.5)

# use imexam to interactively get approx coors for fiducial star

             print ( "")
             print ( "  >>>>> Identify some stars  ( a  q )  " )
             print ( "")

	    imexamine ( keeplog+,logfile='slist')
#           imexamine ( rootname,1,keeplog+,logfile='slist' )
           !egrep -v '#' slist | awk '{print $1, $2 }' > init.dat
										    }

# feed imexam co-ors to center task
# use center task to get refined co-ors of fiducial star;

# make sure to set centerpars and datapars used by center task as follows:
  center.verify = no
  centerpars.calgori="centroid"
  centerpars.cbox=2.5
  centerpars.cthresh=0
  centerpars.minsnra=1
  centerpars.cmaxite=10
  centerpars.maxshif=1
  centerpars.clean=no
  centerpars.rclean=1
  centerpars.rclip=2
  centerpars.kclean=3
  centerpars.mkcente=no
  datapars.scale=0.125
  datapars.fwhmpsf=2.5
  datapars.emissio=yes
  datapars.sigma=INDEF
  datapars.datamin=INDEF
  datapars.datamax=INDEF
  datapars.noise="poisson"
  datapars.ccdread=""
  datapars.gain=""
  datapars.readnoi=0
  datapars.epadu=1
  datapars.exposur=""
  datapars.airmass=""
  datapars.filter=""
  datapars.obstime=""
  datapars.itime=1


	   center (rootname, coords="init.dat", output="results.dat", interactive=no, radplots=no, verify=no, update=no) 

	   !egrep -v '#' results.dat > log.dat
	   !egrep -v 'isx' log.dat >xy.dat
	   !awk '{print $1, $2}' xy.dat > xycoor1.dat


#	x1, x2 are coors of fiducial star in zeropt ref frame
#	Store psf values in 'isx' and 'idf' frame headers

	pclear ( "psf0" ) ; pclear ( "psf1" )

#	imexamine (keeplog+,logfile="psf0",imagecur="xycoor1.dat" )
	imexamine ( rootname,1,keeplog+,logfile="psf0",imagecur="xycoor1.dat" )
           !egrep -v '#' psf0 | awk '{print $13, $15 }' > psf1

	   list2 = "" ; list2 = "xycoor1.dat"
	   while (fscan (list2, x1, y1) !=EOF) { }

	   list2 = "" ; list2 = "psf1"
	   while (fscan (list2, psf_e, psf_d) !=EOF) { }

#	   write offsets and ref file name to header of ref file

           prename = substr(filename,1,9)
	   root = substr(filename, 5,9)

	   parkey (0, prename, "XOFF", add+)
	   parkey (0, "idf_"//root, "XOFF", add+)
	   parkey (0, prename, "YOFF", add+)
	   parkey (0, "idf_"//root, "YOFF", add+)
	   parkey (zptfile, prename, "REFERENC", add+)
	   parkey (zptfile, "idf_"//root, "REFERENC", add+)

	   parkey (psf_e, prename, "PSFE", add+)
	   parkey (psf_e, "idf_"//root, "PSFE", add+)
	   parkey (psf_d, prename, "PSFD", add+)
	   parkey (psf_d, "idf_"//root, "PSFD", add+)

	   pclear ("results.dat")
	   pclear ("xy.dat")
           pclear ("log.dat")
	   pclear ("init.dat")

	   }   #closing brace for if i==1 loop 
	      
	# For files after first, calc coords for center by
	# adding offset to coordinates x1, y1
        
	else if (i > 1)
	   {

#read offset values from header

	   keypar (filename, "asecs", silent+)
	   rasec = real(keypar.value) 
	   keypar (filename, "dsecs", silent+)
	   decsec = real(keypar.value) 
	   
# calc offset w/r/t ref frame location, find newest coors for fid star

# Need to fix up putting in xstart and ystart variable
           xrasec = real (rasec)
           ydecsec = real (decsec)
	   
# read ref file coors from center output
           list2 = "" ; list2 = "xycoor1.dat"
           while (fscan (list2, x1, y1) !=EOF) {

# calculate new estimated coors in pixels
# xrasec - first_asec is delta in asecs
	   newxinit = x1 + sense * (xrasec -  first_asec) / -0.126
	   newyinit = y1 - sense * (ydecsec - first_dsec) / 0.126

	   print (newxinit, " ", newyinit ,>> "newinits.dat")

           }
# display image file

           rootname = substr(filename,1,9)
           prename = substr(filename,1,9)

           display (rootname, 1, fill+, zs-,zr-,z1=-200,z2=200, xmag=0.5, ymag=0.5)
           tvmark (1, coords="newinits.dat", mark="circle", radii="15",color=205)

# run center 2 times- 1)for general location, 2)to refine using smaller search region
           centerpars.cbox = 5.

	   center (rootname, coords="newinits.dat", output="results.dat", interactive=no, radplots=no, verify=no, update=no) 

	   !egrep -v '#' results.dat > log.dat
           !egrep -v 'fits' log.dat >xy.dat
	   !awk '{print $1, $2}' xy.dat > xycoor.dat

	   # set smaller search region
           centerpars.cbox = 2.5

            pclear("results.dat")
            pclear("xy.dat")
            pclear("log.dat")

           center (rootname, coords="xycoor.dat", output="results.dat", interactive=no)

           pclear("xycoor2.dat")

           !egrep -v '#' results.dat > log.dat
           !egrep -v 'isx' log.dat >xy.dat
           !awk '{print $1, $2}' xy.dat > xycoor2.dat

# 	xycoor2.dat contains center task's final coors for star
#	Get psfs

	pclear ( "psf0" ) ; pclear ( "psf1" )

	imexamine (keeplog+,logfile="psf0",imagecur="xycoor2.dat" )
#	imexamine ( rootname,1,keeplog+,logfile="psf0",imagecur="xycoor2.dat" )
           !egrep -v '#' psf0 | awk '{print $13, $15 }' > psf1

	   list2 = "" ; list2 = "psf1"
	   while (fscan (list2, psf_e, psf_d) !=EOF) { }

# show center task's coors on display
           tvmark (1, coords="xycoor2.dat", mark="circle",color=204, radii="10")

!          paste xycoor1.dat xycoor2.dat >xycoor3.dat
# subtract new coors from ref file coors to get offset for each fiducial star
!          awk '{print ($1-$3), ($2-$4)}' xycoor3.dat > xycoor4.dat


# offset = zeropt frame location - current frame location
# calc average offset for all fiducials; xycoor4.dat is offsets list

           k = 0.0
           xsum = 0.
           ysum = 0.
           list2 = "" ; list2 = "xycoor4.dat"
	   while (fscan (list2, x2, y2) !=EOF) { 
           k = k + 1.0
           xsum = xsum + x2
           ysum = ysum + y2
           }
           
           x3 = xsum / k
           y3 = ysum / k

           pclear("init.dat")
           pclear("xycoor.dat")
           pclear("newinits.dat")
           pclear("results.dat")

# write offset (in pix) and name of zerpoint REFERENC file into header 
           prename = substr(filename,1,9)
	   root = substr(filename,5,9)

	   parkey (x3, prename, "XOFF", add+)
	   parkey (x3, "idf_"//root, "XOFF", add+)
	   parkey (y3, prename, "YOFF", add+)
	   parkey (y3, "idf_"//root, "YOFF", add+)
	   parkey (zptfile, prename, "REFERENC", add+)
	   parkey (zptfile, "idf_"//root, "REFERENC", add+)
	   parkey (psf_e, prename, "PSFE", add+)
	   parkey (psf_e, "idf_"//root, "PSFE", add+)
	   parkey (psf_d, prename, "PSFD", add+)
	   parkey (psf_d, "idf_"//root, "PSFD", add+)

	   }	  # close i>1 loop

	}	#close while loop used by fscan; fscan goes on to next file in list


  pclear("init.dat")
  pclear ("slist")
  pclear ("xycoor.dat")
  pclear ("newinits.dat")
  pclear ("results.dat")
  pclear ("xy.dat")
  pclear ("log.dat")

# call reset to fix offsets so that offset scale starts at zero

  print ("resetting offsets to positive values")

  p_reset (filelist)

# final cleanup of temp files

  pclear ("init.dat")
  pclear ("slist")
  pclear ("xycoor1.dat")
  pclear ("xycoor2.dat")
  pclear ("xycoor3.dat")
  pclear ("xycoor4.dat")
  pclear ("newinits.dat")
  pclear ("results.dat")
  pclear ("xy.dat")
  pclear ("log.dat")

end
