procedure p_engr ( pair_list )

string pair_list
struct *list1
struct *list2


#  Stores PSFs and Sky+Tel Backgrounds engineering data 
#  for PANIC frames.
#
#	Definitions
#	
#	psfe     FWHM enclosed  pixels
#	psfd	 FWHM direct    pixels
#	
#	psfea, psfda      FWHMs  arcsec
#	
#	psfeaz, psfdaz    FWHMs  arcsec at zenith (theory: /**0.6)
#
#	psfeazk, psfdazk  FWHMs  arcsec at zenith (theory: /**0.6) at Ks (theory: **0.2)
#
#	The scale is 0.126"/pixel; 5log10(1./scale) = 4.498
#
#	sky           ADU/sec   as measured
#	skyz          ADU/sec   at zenith for YJH (theory: /airmass)
#	sky_mag       sky       in mag per square arcsec as measured
#	skyz_mag      skyz      in mag per square arcsec at zenith
#
#	For Ks "sky" is not expected to go linearly with airmass, nor is it 
#	corrected for airmass. 

begin
	string fname, onumber, night, object, filter, idf, date, sp
	string onumber1, onumber2, psf_out, bck_out
	real airmass, etime, sensor1, frmedian, t_amb
	real psfe, psfd, psfea, psfda, psfeaz, psfdaz, psfeazk, psfdazk
	real exp_psf, f1, f2, sky, skyz, sky_mag, skyz_mag
	real ny, nj, nh, nk, ngt, zmag, dmag_air
	int i, p1, p2, lctime, time, nloops

	sp = "  "

	list1 = "" ; list1 = pair_list
	while ( fscan ( list1, p1, p2 ) != EOF )	{

	padindex(p1) ; onumber1 = padindex.output
	padindex(p2) ; onumber2 = padindex.output

	pclear ( "eng_file1" ) ; pclear ( "eng_file2" )

	  for ( i = p1; i <= p2 ; i += 1 )	{

	padindex(i) ; onumber = padindex.output
	fname = "idf_"//onumber  

	if ( access ( fname//".fits"))	{

	if ( i == p1 )		{
	hselect ( fname,"night,object,filter,nloops,exptime", "yes", >> "eng_file1" )

	list2 = "" ; list2 = "eng_file1"
	while ( fscan ( list2, date,obj,filter,nloops,etime) != EOF )	{ }

	  if ( filter == "Yc" )  f2 = 1.159
	  if ( filter == "Jc" )  f2 = 1.116
	  if ( filter == "Hc" )  f2 = 1.059
	  if ( filter == "Ks" )  f2 = 1.000

				}

	hselect ( fname,"$I,lc-time,airmass,sensor-1,frmedian,psfe,psfd", "yes", >> "eng_file2" )
					}
						}

	p_get_ngt ( date )
	
	list2 = "" ; list2 = "ngt_con"
	while ( fscan ( list2, ny, nj, nh, nk ) != EOF ) 	{}

	  if ( filter == "Yc" )  ngt = ny
	  if ( filter == "Jc" )  ngt = nj
	  if ( filter == "Hc" )  ngt = nh
	  if ( filter == "Ks" )  ngt = nk


	psf_out = "psf_"//date//"_"//p1//"_"//p2
	bck_out = "bck_"//date//"_"//p1//"_"//p2
	pclear ( psf_out ) ; pclear ( bck_out )

	exp_psf = etime*nloops
	printf ("%s%s%s%s%s%8.0f\n",date,sp,object,sp,filter,exp_psf, > psf_out )
	printf ("%s%s%s%s%s\n",date,sp,object,sp,filter, > bck_out )

	list2 = "" ; list2 = "eng_file2"
	while ( fscan ( list2, idf, time, airmass, t_amb, frmed, psfe, psfd) != EOF ) 	{

#	PSF parameters

	psfea = psfe*0.126 ; psfda = psfd*0.126 
	f1 = airmass**0.6
	psfeaz = psfea/f1 ; psfdaz = psfda/f1 ; psfeazk = psfeaz/f2 ; psfdazk = psfdaz/f2

	printf ( "%s%10.0f%8.3f%8.2f%6.2f%8.2f%6.2f%8.2f%6.2f\n", idf,time,airmass,psfea,psfda,psfeaz,psfdaz,psfeazk,psfdazk, >> psf_out )

#	Background parameters

          if ( filter == "Yc" ) dmag_air = 0.10*(airmass - 1.0)
          if ( filter == "Jc" ) dmag_air = 0.12*(airmass - 1.0)
          if ( filter == "Hc" ) dmag_air = 0.04*(airmass - 1.0)
          if ( filter == "Ks" ) dmag_air = 0.08*(airmass - 1.0)

	zmag = 20.000 + 1.0857*log(etime) - dmag_air - ngt

	sky      = frmed/etime
	skyz     = sky/airmass
	sky_mag  = zmag -1.0857*log(frmed) - 4.498
	skyz_mag = zmag -1.0857*log(frmed/airmass) - 4.498

	if ( filter == "Ks" )	{

	printf ( "%s%10.0f%8.3f%8.1f%10.0f%8.2f\n", idf,time,airmass,t_amb,sky,sky_mag, >> bck_out )

				}	else	{

	printf ( "%s%10.0f%8.3f%8.0f%8.0f%8.2f%8.2f\n", idf,time,airmass,sky,skyz,sky_mag,skyz_mag, >> bck_out )
						}




											}

							}

end



