procedure p_fid ( p1, p2, autman )

int p1      {prompt="First frame"} 
int p2      {prompt="Last frame"} 
string autman   {"aut",enum="aut|man",prompt="Automatic or manual star selection?"}
struct *list1

#  Used during the night to loop-combine 'irp'
#  frames, find median sky, subtract it, and 
#  display the first resulting frame for 
#  identifying the fiducial star. Will use 'idf'
#  frames if they exist.

begin
   string pre, onumber, isx_frame
   real xfid, yfid, frm_med, sky_med, ratio
   real mag, elli, fwhm, fhthr, ellithr, fhtmin
   int flag

   padindex(p1); onumber = padindex.output
   pclear ("i_list")

   fhtmin = 2.5
   fhthr = 10.0
   ellithr = 0.2

   if ( access ("idf_"//onumber//".fits")) {
      p_pre_list ( "idf", p1, p2 )
      copy ( "idf_list", "i_list" )
      pre = "idf_"
   } else {
      p_loops_combine ( "irp", p1, p2 )
      p_pre_list ( "icp", p1, p2 )
      copy ( "icp_list", "i_list" )
      pre = "icp_"
   }

#  Now have 'pre' frames p1 through p2, and a list 'i_list'.
#  'pre' is either 'icp_' or 'idf_'. 
#  This next section is from 'p_sky1', but only does first frame.
#  The sky is saved in case it is need for crowded frame IDs. 

   pclear ("sky.fits") ; pclear ( "sky_"//onumber)
   imcombine ("@"//"i_list","sky.fits", combine="median", scale="mode",masktyp="none")
   p_store ( "sky" )
   keypar("sky", "FRMEDIAN",silent+)
   sky_med = real(keypar.value)
   imcopy ( "sky", "sky_"//onumber)

#   Scale the sky frame to the median value of the first frame and subtract.

   padindex (p1) ; onumber=padindex.output
   p_store ( pre//onumber )
   keypar( pre//onumber, "FRMEDIAN",silent+)
   frm_med = real(keypar.value)
   ratio = frm_med / sky_med

   isx_frame = "isx_"//onumber
   pclear (isx_frame )

   pclear ( "dummy" )
   imar ( "sky.fits", "*", ratio, "dummy" , divz=1., pixtype="real", calctyp="real", verbose-)
   imar ( pre//onumber, "-", "dummy", isx_frame, divz=1., pixtype="real", calctyp="real", verbose- )
   pclear("sex_frame")
   imcopy(isx_frame, "sex_frame", ver-)

# If on automatic mode: Call sextractor.
# Make a list of objects, sort it by decreasing magnitude, select the first 
# object with 'flag = 0' and far enough from the borders:

	if (autman == "aut") {
	  pclear ("sex.cat"); pclear ("init.dat")

	  !sex sex_frame.fits -c default_fid.sex
	  !egrep -v '#' sex.cat | sort -g > init.dat

	  list1 = "" ; list1 = "init.dat"
	  while ( fscan ( list1, mag, xfid, yfid, elli, fwhm, flag ) != EOF ) { 

	    if (flag == 0 && xfid > 80 && xfid < 944 && yfid > 80 && yfid < 944 \
            && elli < ellithr && fwhm < fhthr && fwhm > fhtmin ) goto found
	  }
	}
# If no suitable FID star was found, or on manual mode, ask the user to mark one:

   pclear ("init.dat") ; pclear ("slist") 
   display (isx_frame, 1, erase-, bor+, fill+, zs+, xmag=0.5, ymag=0.5)
                                                                                                                      
#    Use imexam to interactively get approx coords for fiducial star
#   This could be generalized to a list of stars if we wanted to keep 
#   these files of XY values around until pipeline is used. 
                                                                                                                      
   print ( "")
   print ( "  >>>>> Identify a star  ( a  q )  " )
   print ( "")
                                                                                                                      
   imexamine ( isx_frame,1,keeplog+,logfile='slist' )
   !egrep -v '#' slist | awk '{print $1, $2 }' > init.dat

   list1 = "" ; list1 = "init.dat"
   while ( fscan ( list1, xfid, yfid ) != EOF )   { }

found:	
	pclear ("slist")
 	display (isx_frame, 1, erase-, bor+, fill+, zs+, xmag=0.5, ymag=0.5)
        print (xfid, yfid, > "slist")
	tvmark (1,"slist", mark="circle", radii=10, color=205, interac-)
	tvmark (1,"slist", mark="circle", radii=11, color=205, interac-)

	pclear ( isx_frame )
   if ( pre == "icp_" )   {
      parkey ( xfid, "irp_"//onumber//"_001.fits", "XFID", add+) 
      parkey ( yfid, "irp_"//onumber//"_001.fits", "YFID", add+)
   } else {
      parkey ( xfid, "idf_"//onumber//".fits", "XFID", add+) 
      parkey ( yfid, "idf_"//onumber//".fits", "YFID", add+)
   }
   pclear("i_list")
   pclear("dummy"); pclear("sky"); pclear("init.dat")
   pclear("slist"); pclear("sex_frame");  pclear("sex.dat")
end
