procedure p_fix_cr ( picin )

string picin
struct *list1

#  Fixes cosmicrays on a pair (or three) of 'izp' images.
#  Called from within 'p_loopavg_sep', when loop=2 or 3.
#  'pic' is of the form   
#
#   pic1 = pre//"_"//onumber//"_001.fits" OR
#          pre//"_"//onumber//"_001"
#
#  The algorithm is:
#
#  p12 = pic1 - pic2   ;   p12 is corrected in place for crs
#  p21 = pic2 - pic1   ;   p21 is corrected in place for crs
#
#  pic1(corr) = p12 + pic2
#  pic2(corr) = p21 + pic1
#
#  The sigma for detection of crs is 2.5*sigma on the differenced 
#  frame, which is 3.5*sigma on a raw frame. 
#  
 
begin
   string pic1, pic2, onumber, msk1, msk2, msk3
   real sigma, s3, level

   pclear ( "p12" ) ; pclear ( "p21" ) ; pclear ( "imstatout" )
   pclear ( "sig" ) ; pclear ( "pic1c" ) ; pclear ( "pic2c" )

   pic1 = substr(picin,1,9)//"_001"
   pic2 = substr(picin,1,9)//"_002"
   onumber = substr(picin,5,9)
   msk1 = "msk_"//onumber//"_001"
   msk2 = "msk_"//onumber//"_002"
   msk3 = "msk_"//onumber//"_003"

   if ( access ( pic1//".fits" ) && !access ( pic2//".fits" ))   {

      imstat ( pic1, fields="stddev,midpt", >  "imstatout" )
      tail ( "imstatout", nl=1, > "sig" ) 

      list1 = "" ; list1 = "sig"
      while ( fscan ( list1, sigma, level ) != EOF ) { }
      s3 = 5.*sigma 

      pclear ( msk1 ) ; pclear ( msk2 )

      cosmicrays ( pic1, pic1, thresho=s3, interac-, answer-, crmasks=msk1, fluxrat=10.)
      hedit (pic1, "BPM", msk1 ,add+,verify-)
   }

   if ( access ( pic1//".fits") && access ( pic2//".fits" ))   {

      imarith ( pic1, "-", pic2, "p12" )
      imarith ( pic2, "-", pic1, "p21" )
      imstat ( "p12", fields="stddev,midpt",lower=-400,upper=400, >  "imstatout" )
      tail ( "imstatout", nl=1, > "sig" ) 
   
      list1 = "" ; list1 = "sig"
      while ( fscan ( list1, sigma, level ) != EOF ) { }
      s3 = 4.*sigma 
   
      pclear ( msk1 ) ; pclear ( msk2 )
   
      cosmicrays ( "p12", "p12", thresho=s3, interac-, answer-, crmasks=msk1, fluxrat=10.)
      cosmicrays ( "p21", "p21", thresho=s3, interac-, answer-, crmasks=msk2, fluxrat=10.)
   
      imarith ( "p12", "+", pic2, "pic1c" )
      imarith ( "p21", "+", pic1, "pic2c" )
   
      pclear ( pic1 ) ; pclear ( pic2 )
      imrename ( "pic1c", pic1 )
      imrename ( "pic2c", pic2 )

      hedit (pic1, "BPM", msk1 ,add+,verify-)
      hedit (pic2, "BPM", msk2 ,add+,verify-)

   }  else {}
   

#  Now if loop = 3, correct that one using pic2

   pic1 = substr(picin,1,9)//"_003"

   if ( access ( pic1//".fits") && access ( pic2//".fits" ))   {

      pclear ( "p12" ) ; pclear ( "sig" ) ; pclear ( "imstatout" )
      imarith ( pic1, "-", pic2, "p12" )
      imstat ( "p12", fields="stddev,midpt", >  "imstatout" )
      tail ( "imstatout", nl=1, > "sig" ) 
   
      list1 = "" ; list1 = "sig"
      while ( fscan ( list1, sigma, level ) != EOF ) { }
      s3 = 4.*sigma 
      pclear ( msk3 )
   
      cosmicrays ( "p12", "p12", thresho=s3, interac-, answer-, crmasks=msk3, fluxrat=10. )
      pclear ( "pic1c" )
      imarith ( "p12", "+", pic2, "pic1c" )
   
      pclear ( pic1 ) 
      imrename ( "pic1c", pic1 )
      hedit (pic1, "BPM", msk3 ,add+,verify-)

   }  else {}
 
end



   
   
   
