procedure p_flatten ( n1, n2 )

int n1, n2
struct *list1

#  Flattens 'icd' frames to produce 'idf' frames.
#
#  Assumes that 'icd' linearized, loop-combined, and 
#  dark-subtracted frames exist. 
#  'n1' and 'n2' are the starting and ending 
#  frame numbers of a set. Frames n1 and n2 must exist,
#  but those in between need not. 
#  It is assumed that 'chk_headers' and 'fix_headers' have 
#  been run and that therefore all obstypes are correct. 
#  Only 'dark' obstype frames are NOT processed. 
#
#  Flats must exist. The nomenclature is:
#	
#   npdflatJc_28nov2003.fits  (normalized/panic/dome/flat/filter/_date)
#	    
#  (if it exists). If not, the twi rather than dome flat is used)
#  If this does not exist, then a canonical flat is taken from 
#  the flat storage directory. 
#
#  Rev 30aug2004 SEP: Now add the two mask files 'mask_date.pl' and 
#  mask_NNNNN.pl and store in header. 
#
#
#



begin
	string filter, obstyp, date, flat_path, flat, dflat, tflat, sflat
	string filter1, filter2
	string icd_frame, idf_frame, flat_now, onumber, msk
	real fr, frmedian

	for ( i = n1 ; i <= n2; i += 1 )	{

#  Get filter, night, and obstyp out of header of each frame in 
#  the n1,n2 sequence

	padindex(i) ; onumber = padindex.output ; icd_frame = "icd_"//onumber//".fits"

	if ( access ( icd_frame )) 	{

#  Decode the filter combination. This copied from 'p_loopavg_sep.cl'
#  These are current as of 17 June 2004.

	keypar(icd_frame,"FILTER1",silent+)
	filter1 = str(keypar.value)

	keypar(icd_frame,"FILTER2",silent+)
	filter2 = str(keypar.value)

	if ( filter1 == "o1" && filter2 == "Yc" ) filter = "Yc"
	if ( filter1 == "o1" && filter2 == "Jc" ) filter = "Jc"
	if ( filter1 == "o1" && filter2 == "Hc" ) filter = "Hc"
	if ( filter1 == "o1" && filter2 == "Ks" ) filter = "Ks"
	if ( filter1 == "o1" && filter2 == "Bg" ) filter = "Bg"
	if ( filter1 == "o1" && filter2 == "dk" ) filter = "dk"

	if ( filter1 == "h2" && filter2 == "op" ) filter = "h2"
	if ( filter1 == "HA" && filter2 == "op" ) filter = "HA"
	if ( filter1 == "HB" && filter2 == "op" ) filter = "HB"
	if ( filter1 == "HC" && filter2 == "op" ) filter = "HC"
	if ( filter1 == "HD" && filter2 == "op" ) filter = "HD"
	if ( filter1 == "h2" && filter2 == "Jc" ) filter = "h2"

	hedit(icd_frame,"FILTER",filter,add+,update+,verify-)

	  keypar(icd_frame, "FILTER",silent+)
	  filter = str(keypar.value)

	  keypar(icd_frame, "NIGHT",silent+)
	  date = str(keypar.value)

	  keypar(icd_frame, "OBSTYPE",silent+)
	  obstyp = str(keypar.value)

#  All but dark frames are flattened. The flattened dome flats 
#  are best for computing badpixel masks. 

	if ( obstyp == 'astro' || obstyp == 'standard' )	{

	  idf_frame = "idf_"//onumber
	  pclear ( idf_frame )

#  Select flat in priority order: dome, twi, stored

	    dflat = "npdflat"//filter//"_"//date//".fits"
	    tflat = "nptflat"//filter//"_"//date//".fits"

	flat_path = "home$panic_files/flats/"

	    sflat = flat_path//"sptflat"//filter//".fits"

#  Compute, store, and retrieve the icd frame median to use for 
#  the divzero parameter.

	p_store ( icd_frame )

	keypar(icd_frame, "FRMEDIAN",silent+)
	frmedian = real(keypar.value)

	if ( access ( tflat ))	flat = tflat
#	if ( access ( dflat ))	flat = dflat

	if ( !access ( dflat ) && !access ( tflat ))	flat = sflat

	  imarith ( icd_frame, "/", flat, idf_frame, divzero=frmedian )
          hedit (idf_frame,"FLAT", flat,update+,add+,verify-)
	  hedit (idf_frame,"ZMAG",26.00,update+,add+,verify-)

#  ***********************************************************************
#	No pixel value can be outside original range of +- 32768/flat(x,y).
#	Consistent with the allowed range in the flat-making routines, we
#	allow flat to go as low as +0.5. There is no way for a valid 
#	pixel value to be < -500 in an icd frame, so set lower limit to -1000.
#	There could be a valid icd pixel value near 32000 on top of a valid
#	flat(x,y) ~ 0.5, so set upper limit to 64000.
#	Any outside this range have been created by spurious values in the
#	flat.
	
	  imreplace ( idf_frame, value=frmedian, lower=INDEF, upper=-1000 )
	  imreplace ( idf_frame, value=frmedian, lower=64000, upper=INDEF  )

#  Store mask names in header

  msk = "mcp_"//onumber//".pl"

  if (access ( msk ))	{
   imarith ( msk,"+","mask_"//date//".pl", msk, pixtype="int",calctype="int")
   imreplace ( msk, value=1, lower=1, upper=INDEF )
			} else  {
   imcopy ( "mask_"//date//".pl", msk )
				}
   parkey ( msk, idf_frame, "BPM", add+)

#	Fix frame edges, which stores the median value of the frame in its header.
#

	p_edges ( idf_frame )

													}
							}
									}
end















