procedure p_fx_c_offsets (p1, p2, p3, p4, p5, p6)

#############################################################
#  Written by P McCarthy and J Wilson, 8/18/99              #
#  Modified by K Koviak for PANIC 10/28/03                  # 
#  Finds offsets of each frame relative to a "reference     #
#  frame" within a dither sequence and writes them          #
#  into the header as XOFF and YOFF, in pixels.             #
#  NOTE: Must start display tool before running this task   #
#                                                           #
#############################################################

# This version handles the case where we have crowded fields
#  and so need to use p_sky3 rathern than p_sky1  --CRB

int p1, p2, p3, p4, p5, p6
struct          *list1, *list2

begin

string    filelist, filename, root
string    prename, rootname, anothername, type
int       i
file      init.dat, slist
real      x, y, x0, y0, new_asecs, new_dsecs
real      xstart, ystart, first_asec, first_dsec
real      first_ra, first_dec, rastart, decstart
real      raref, decref, x2, y2
real      xref, yref, x_pred, y_pred, rotang, sense


# First, we assume that the off-source skies need to be fixed too
p_fx_offsets (p3, p4)
p_sky1 ( "idf", p3, p4 ) 
if ( p3 != p5 && p4 != p6 ) {
   p_fx_offsets (p5, p6)
   p_sky1 ( "idf", p5, p6 ) 
}

# Now make a good sky
p_sky3 (p1, p2, p3, p4, p5, p6)

p_pre_list ( "isx", p1, p2 )
filelist = "isx_list"


# initialize variables
i = 0                #counting variable
xstart = 0.0                
ystart = 0.0                
new_asecs  = 0.0
new_dsecs  = 0.0
first_asec = 0.0
first_dsec = 0.0
first_ra = 0.0
first_dec = 0.0
x_pred = 0
y_pred = 0
pclear ("slist")
pclear ("init.dat")


# loop through files calculating offset and new approx co-ors for fiducial;

list1 = "" ; list1 = filelist
while (fscan (list1,filename) != EOF)  {
   
   # read file header
   
   keypar(filename,"ROTANG",silent+)
   rotang = real(keypar.value)
   
   if ( rotang <= 50. ) sense = -1.
   if ( rotang >= 200. ) sense = 1. 
   
   keypar (filename, "asecs", silent+)
   xstart = real(keypar.value)
   keypar (filename, "dsecs", silent+)
   ystart = real(keypar.value)
   keypar (filename, "ra", silent+)
   rastart = real(keypar.value)
   keypar (filename, "dec", silent+)
   decstart = real(keypar.value)
   
   root = substr(filename,5,9)
   rootname = substr(filename,1,9)
   prename = substr(filename,1,9)
    
   # Calculate difference between coords in ref frame and in this frame
   xref = int(xstart - first_asec)    #ref frame coors - this frame coors
   yref = int(ystart - first_dsec)    #ref frame coors - this frame coors
   raref = real(rastart - first_ra)
   decref = real(decstart - first_dec)
   
   print ("xstart = ",xstart," ystart = ",ystart)
   
   
   # determine scale of coor change to see if dither or 
   # if pointing has changed w/r/t previous frame
   
   # if ( xref >=  1)  i = 0
   # if ( xref <=  -1) i = 0
   # if ( yref >=  1)  i = 0
   # if ( yref <=  -1) i = 0
   
   
   # if ( raref >=  0.0400000)  i = 0
   # if ( raref <=  -0.0400000) i = 0
   # if ( decref >= 0.0400000)  i = 0
   # if ( decref <= -0.0400000) i = 0
    
   print ("raref = ",raref, "decref = ",decref)
   print ("xref = ",xref, " yref = ",yref)
   print ("i = ",i)
   
   # if (xstart==0 && ystart==0) i = 0
         
   if (i == 0)   {
                 
      pclear ("init.dat")
      pclear ("slist")
      
      # Save ref file coords in first_ variables
      first_asec = xstart
      first_dsec = ystart
      first_ra = rastart
      first_dec = decstart
      
      rootname = substr(filename,1,9)
      prename = substr(filename,1,9)
      print ("file to display= "//prename)
      display (prename, 1, erase+, bor+, fill+, zs+, xmag=0.5, ymag=0.5)
      
      
      # use imexam to interactively give approx coords for fiducial star
      print ( "")
      print ( "  >>>>> Identify a star  ( a  q )  " )
      print ( "")
      
      pclear ("slist")
      imexamine ( rootname,1,keeplog+,logfile='slist' )
      !egrep -v '#' slist | awk '{print $1, $2 }' > init.dat
      !/usr/bin/tail init.dat
      pclear ("slist") 
      
      #  if asecs & dsecs = 0,0 set new x0 and y0
      # read star's coords from imexam output files
      list2 = "" ; list2 = "init.dat"
      while (fscan(list2,x,y) !=EOF) {
         x0 = x
         y0 = y
      }  # close while for fscan
         
   }  # close if i == 0
   
   else {
      display (rootname, 1, fill+, zs+, xmag=0.5,ymag=0.5)
      print ( "")
      print ( "  >>>>> Now identify the SAME star  ( a  q )  " )
      print ( "")
   
      pclear ("slist")
      pclear ("init.dat")
      imexamine ( rootname,1,keeplog+,logfile="slist" )
      !egrep -v '#' slist | awk '{print $1, $2 }' > init.dat
      !/usr/bin/tail init.dat
      list2 = "" ; list2 = "init.dat"
      while (fscan(list2,x,y) !=EOF) { }
   
      # save old asecs, dsecs in headers before updating w/r/t ref file
   
      parkey (xstart, prename, "old_asecs", add+)
      parkey (xstart, "idf_"//root//".fits", "old_asecs", add+)
      parkey (ystart, prename, "old_dsecs", add+)
      parkey (ystart, "idf_"//root, "old_dsecs", add+)
   
   
      # Calculate predicted coords for x, y; x0, y0 are in ref frame
      print ("first_asec = ",first_asec," first_dec =",first_dsec)
      print ("xstart = ",xstart," ystart =",ystart)
      print ("x0 =", x0, "y0 = ",y0)
      x_pred = x0 - sense * ((xstart - first_asec)/0.125)
      y_pred = y0 - sense * ((ystart - first_dsec)/0.125)
      
      print ("x_pred = "," ", x_pred, "x = "," ", x)
      print ("y_pred = "," ", y_pred, "y = "," ", y)
   
      # Calculate adjusted asecs, dsecs
      new_asecs = xstart - sense * ((x - x_pred) * 0.125)
      new_dsecs = ystart - sense * ((y - y_pred) * 0.125)
   
      print ("file name is ",prename)
      print (xstart," ",new_asecs," ",ystart," ",new_dsecs)
   
   
      #   update headers in both 'isx' and 'idf' frames
   
      parkey (new_asecs, prename, "asecs", add+)
      parkey (new_asecs, "idf_"//root, "asecs", add+)
      parkey (new_dsecs, prename, "dsecs", add+)
      parkey (new_dsecs, "idf_"//root, "dsecs", add+)
   
   }   # close else for i != 0
              
   i = i + 1
   
}   # close while

# clean up
# Now run p_dither

p_dither ( "isx", p1, p2 )

pclear ("init.dat")
pclear ("slist")

end
