#include<stdlib.h>
#include<math.h>
#include<stdio.h>
#include "rw_fits.h"
/*#include<debugs.h>*/
#include<string.h>

/* Compiling the code:
 *
 * gcc -o p_linearize p_linearize.c rw_fits.c -lm
 *
 * Usage:  p_linearize inst n1 n2
 *         where inst = p for panic
 *                    = r for retrocam (but not setup for this)
 *         n1 and n2 are the starting and ending sequence numbers,
 *         respectively.  */

/* #  /home/persson/iraf/panic_scripts/p_linearize.cl

#  PANIC (camera = p) or RetroCam (r) linearization script. 
#  Linearizes 'irp' or 'irr' frames n1 through n2, skipping 
#  missing ones, and producing 'izp' or 'izr' frames. Parameters
#  and basic algorithm from Paul Martini.
#  Rev:  30nov2003   SEP
#
#  Set readout time parameters for each camera
#  All times in seconds.
#
#  reads_ep = number of end-point reads
#  pre_ep = number of pre-endpoint reads; set to 1 for all cameras.
#
#  tf = frame readout time
#  td = delay time
#  tfd = tf + td
#
#  ta = acquisition time
#  t_pre_ep = pre-endpoint readout time = pre_ep * tfd 
#  t_read = total effective frame readout time = reads_ep * ( tfd + ta ) */


int p_lincor_sep (float *data, int len, float p, float c1, float c2, float c3, 
                  float linthresh) {
   float p1, data1, cor, data2, out1, out2;
   int i;
   /*#   Do linearity correction for an image
   #
   #   p = a/(a + exptime), where a is the overhead due
   #   to prereads and reads. The output image is
   #   [(1+p)*pic](corr) - [p*pic](corr)
   #
   #  The linearity correction is a polynomial above a 
   #  threshold. */
   p1 = 1.0 + p;

   for ( i = 0 ; i < len; ++i ) {
      data1 = data[i] * p1;
      cor = c1 + c2*data1 + c3*data1*data1;
      if ( data1 > linthresh ) {
         out1 = cor;
      } else {
         out1 = data1;
      }

      data2 = data[i] * p;
      cor = c1 + c2*data2 + c3*data2*data2;
      if ( data2 > linthresh ) {
         out2 = cor;
      } else {
         out2 = data2;
      }
 
      data[i] = out1 - out2;
   }
   return (1);
}

int main(int argc, char **argv) {

   float pre_ep;
   int n1, n2;
   char *camera;
   /* char *filename, *filename2; */
   char filename[256], filename2[256];
   float td, ta, tf, c1, c2, c3, linthresh;
   float exptime, reads_ep, t_read, delta, p, res;
   char *readmode;
   float tfd, t_pre_ep;
   int i, j;
   int keep_going;
   fits *frame;

   /* frame.other = (struct other_node *)NULL; */

   pre_ep = 1.;
   
   if ( argc < 4 ) {
      fprintf(stderr, "Usage:  p_linearize camera n1 n2\n");
      fprintf(stderr, "        camera can be:\n");
      fprintf(stderr, "           'r' for RetroCam\n");
      fprintf(stderr, "           'p' for PANIC\n");
      exit(1);
   }
   camera = argv[1];
   n1 = atoi(argv[2]);
   n2 = atoi(argv[3]);
   
   if (strcmp(camera,"r") == 0) {
      td = 0.010 ; tf = 2.0 ; ta = 0.085;
   } else if (strcmp(camera,"p") ==0) {
      td = 0.010 ; tf = 2.0 ; ta = 0.085;
      c1 = 4.291443e+01 ; c2 = 9.752524e-01 ; c3 = 1.962545e-06 ; 
      linthresh = 8000.;
   } else if (strcmp(camera,"w") ==0) {
      td = 0.010 ; tf = 2.0 ; ta = 0.085;
      c1 = 4.291443e+01 ; c2 = 9.752524e-01 ; c3 = 1.962545e-06 ; 
      linthresh = 8000.;
   } else {
      fprintf(stderr, "Unknown camera type %s\n", camera);
   }
   
   tfd = tf + td;
   t_pre_ep = pre_ep * tfd;
   
   for ( i = n1 ; i <= n2 ; i++) {
      keep_going = 1;
      j = 1;
      while(keep_going) {
         frame = new_fits();
         snprintf(filename, sizeof(filename), "irp_%05d_%03d.fits", i, j);
         snprintf(filename2, sizeof(filename2), "izp_%05d_%03d.fits", i, j);
         fprintf(stderr, "Processing %s...\n", filename);
         if ( read_fits_file(filename, frame) !=0 ) {
            keep_going = 0;
            fprintf(stderr, "Moving onto next sequence...\n");
         } else {
            exptime = atof(get_tag(frame, "EXPTIME"));
            sscanf(get_tag(frame, "EXPTIME"), "EXPTIME = %f", &exptime);
            readmode = strtok(get_tag(frame, "READMODE"), "'");
            readmode = strtok((char *)NULL, "'");
            if (strcmp(readmode, "Double") == 0)  reads_ep = 1.0;
            if (strcmp(readmode,"Quadruple") ==0)  reads_ep = 2.0;
            if (strcmp(readmode,"Sextuple") ==0)  reads_ep = 3.0;
            if (strcmp(readmode,"Octuple") ==0)  reads_ep = 4.0;
       
            t_read = reads_ep * ( tfd + ta );
       
            /* Linearize */
       
            delta = ( t_pre_ep + t_read / 2. ) ;
            p = delta / ( delta + exptime );
       
            res = p_lincor_sep(frame->data, frame->naxis[0]*frame->naxis[1], p, 
                  c1, c2, c3, linthresh)  ;
            /* frame->bitpix = 16; */
            if ( write_fits_file(filename2, *frame) != 0 ) {
               fprintf(stderr, "Error writing to %s\n", filename2);
            }
            free_fits_struct(frame);
         }
         ++j;
      }
   }
}
