procedure p_loopavg_sep ( pre, number )

string pre
int number    

string output  

#  /home/persson/iraf/panic_scripts/p_loopavg_sep.cl
#  PANIC or RetroCam
#
#  Input image is a linearized 'izp_' one.
#
#  Output image is icp_number.fits and is the pclipped 
#  average of the inputs if nloops >= 4. For nloops <= 3
#  cosmicray cleaning is applied to the difference images.
#
#  Decodes the two filters and adds header keyword.
#
#  Rev:  30nov2003   SEP
#  Rev:  12mar2004   SEP
#  Rev:  16mar2004   SEP   
#  Rev:  22jul2004   SEP cosmicrays

begin

  int nloops
  string out, onumber, pic, pic_list, filter1, filter2, filter
  string out_sigma, out_scale
  string msk, msk1

  pclear ("loopsumtemp")

#  Pad the frame number

  padindex(number) 
  onumber = padindex.output
  pic = pre//"_"//onumber//"_001.fits"

  if ( access ( pic ))	{

#  Get the number of frames in the loop

  keypar(pic,"NLOOPS",silent+)
  nloops = int(keypar.value)

#  Clean cosmicrays in place using 'p_fix_cr' if nloops = 2 or 3.
#  This routine creates 'mcp_*.fits' frames which are also averaged
#  here.

  if ( nloops == 2 || nloops == 3 )	{
	p_fix_cr ( pic )
					}

#  Expand the picture names into a file called 'pic_list'

  p_loop_expand ( pic )

  pic_list = pre//"_"//onumber//"_list"

#  Create output image name and pclear it. 

  out = "icp_"//onumber//".fits"    ;   pclear ( out )
  out_scale = "icp_"//onumber//"_scale.fits"    ;   pclear ( out_scale )
  out_sigma = "icp_"//onumber//"_sigma.fits"    ;   pclear ( out_sigma )

    if (nloops < 4) { 
      imcombine("@"//pic_list, output=out,rejmask="", headers="", sigma="", combine="average",reject="none",offsets="none", masktyp="none",project=no,pclip=-0.5,scale="none")
    } else {
      imcombine("@"//pic_list, output=out, headers="",rejmask="", sigma="", combine="average",reject="pclip",offsets="none", masktyp="none",project=no,pclip=-0.5,scale="none",lsigma=3,hsigma=3,nkeep=1)
    }

#  Now combine the masks. These are in .pl format at this point. 

  msk = "mcp_"//onumber//".pl"    ;   pclear ( msk )

  if ( access  ( "msk_"//onumber//"_001.pl" ))	{
	imcopy ( "msk_"//onumber//"_001.pl", msk )
						}
  if (  access  ( "msk_"//onumber//"_002.pl" ))   {
	imarith ( "msk_"//onumber//"_002.pl", "+", msk, msk )
						  }
  if (  access  ( "msk_"//onumber//"_003.pl" ))  {
	imarith ( "msk_"//onumber//"_003.pl", "+", msk, msk )
						}

  if ( access ( msk)) imreplace ( msk, value=1, lower=1, upper=INDEF )

#  For loops of many, save the sigma image with scaling turned on; 
#  these are for making bp_masks.

  out_sigma = "icp_"//onumber//"_sigma.fits"    ;   pclear ( out_sigma )

	if ( nloops > 8)	{
   imcombine("@"//pic_list, output=out_scale, headers="",rejmask="", sigma=out_sigma, combine="average",reject="none",offsets="none", masktyp="none",project=no,scale="median")
					} else {}

#  Decode the filter combination.
#  These are current as of 17 June 2004.

	keypar(out,"FILTER1",silent+)
	filter1 = str(keypar.value)

	keypar(out,"FILTER2",silent+)
	filter2 = str(keypar.value)

	if ( filter1 == "o1" && filter2 == "Yc" ) filter = "Yc"
	if ( filter1 == "o1" && filter2 == "Jc" ) filter = "Jc"
	if ( filter1 == "o1" && filter2 == "Hc" ) filter = "Hc"
	if ( filter1 == "o1" && filter2 == "Ks" ) filter = "Ks"
	if ( filter1 == "o1" && filter2 == "Bg" ) filter = "Bg"
	if ( filter1 == "o1" && filter2 == "dk" ) filter = "dk"

	if ( filter1 == "h2" && filter2 == "op" ) filter = "h2"
	if ( filter1 == "HA" && filter2 == "op" ) filter = "HA"
	if ( filter1 == "HB" && filter2 == "op" ) filter = "HB"
	if ( filter1 == "HC" && filter2 == "op" ) filter = "HC"
	if ( filter1 == "HD" && filter2 == "op" ) filter = "HD"
	if ( filter1 == "h2" && filter2 == "Jc" ) filter = "h2"

	hedit(out,"FILTER",filter,add+,update+,verify-)

	}	else { 
	psorry ( pic ) 
}

end

