procedure p_objmask (p1, p2)

#############################################################
#  Written by P McCarthy and J Wilson, 8/21/99 (mod 4/26/02)# 
#  Modified by P McCarthy & K Koviak for PANIC 10/29/03     #
#  Finds objects in averaged frames (the ave_isx files) and #
#  makes object masks (mask_isx files)--one per pointing.   #
#  Task input is a list of all ave_isx*.fits files.	    #
#  Ximtool must be running to use this task.                # 
#							    #
#############################################################

int p1, p2
struct	  *list1, *list2, *list3

begin

string	  filelist, fname, dither_name, old_name, dither_root
string    root, rootnum
file	  sex.cat, sex.list, objects_xy.dat
real      xstar, ystar, xshift, yshift, newx, newy
real      size, x, y
real      r_small, r_medium, r_large, r_vlarge, r_alarge
int       j

#	Make isx list

	p_pre_list ( "isx", p1, p2 )
	filelist = "isx_list"

#       initialize file lists

        r_small  = 5
        r_medium = 10
        r_large  = 15
        r_vlarge = 25
        r_alarge = 45

pclear ("p0.fits") ; pclear ("p1.fits") ; pclear ("p2.fits") ; pclear ("p3.fits") ; pclear ("p4.fits")
pclear ("e0.fits") ; pclear ("e1.fits") ; pclear ("e2.fits") ; pclear ("e3.fits") ; pclear ("e4.fits")
pclear ("e0.pl")   ; pclear ("e1.pl")   ; pclear ("e2.pl")   ; pclear ("e3.pl")   ; pclear ("e4.pl") 

pclear ("e99.pl") ; pclear ("sex_frame.fits") ; pclear("sex.list") ; pclear("sex.cat")
pclear ("objects_xy.dat")

        pclear ("small_objects")
        pclear ("medium_objects")
        pclear ("large_objects")
        pclear ("very_large_objects")
        pclear ("absurdly_large_objects")
      
        old_name = ""

# set imedit parameters
#          imedit.display="no"
#          imedit.autodis="no"
           imedit.default="e"
#          set stdimage=imt2048

  
# display image and call sextractor

#   read the list of ave_isx files
    list1 = "" ; list1 = filelist
    while (fscan (list1,fname) != EOF)  {


#   find name of reference file and dither stack

           keypar (fname,"referenc")
           dither_name = str(keypar.value)
           dither_root = substr(dither_name,1,9)

           root = substr(fname,1,12)
           rootnum = substr(fname,5,9)
#          print ("root="//root)
#          print ("rootnum="//rootnum)

           if (dither_name == old_name) { }
           else {

    
           imcopy ("ave_isx_"//rootnum,"sex_frame")
           display ("sex_frame", 1, zs+, xmag=1, ymag=1)

#	   run sextractor to identify all objects

# !          sex sex_frame.fits -c //default.sex 
!          sex sex_frame.fits -c default.sex 
!          egrep -v '#' sex.cat >sex.list
!          awk '{print $8, $9, $10}' sex.list >objects_xy.dat

#          sort output into three lists according to isophotal size


           list3 = "" ; list3="objects_xy.dat"
           while (fscan (list3,size, x, y) != EOF)  {
              if (size <=  20) print (x,"",y, >> "small_objects") 
              else if (size <= 100) print (x,"",y, >> "medium_objects")
              else if (size <= 400) print (x,"",y, >> "large_objects") 
              else if (size <= 1000) print (x,"",y, >> "very_large_objects") 
              if (size >= 1000) print (x,"",y, >> "absurdly_large_objects")
              print (x,"",y, >> "objects")
}

#	   zero out image to make blank template for mask to go in
           imreplace ("sex_frame.fits",value=0.0,upper=INDEF,lower=INDEF)
           imcopy ("sex_frame.fits","p0.fits")
           imcopy ("sex_frame.fits","p1.fits")
           imcopy ("sex_frame.fits","p2.fits")
           imcopy ("sex_frame.fits","p3.fits")
           imcopy ("sex_frame.fits","p4.fits")
   
            pclear ("mask_isx_"//rootnum//".fits")
            pclear ("mask_isx_"//rootnum//".pl")

#  imedit the frame for large, medium and small objects, after checking that the
#  list is not empty for a particular size bin
#  makes intermediate masks for each size which will be added together later

           if (access ("small_objects")) {
           print ("small objects")
            tvmark (1, coords="small_objects",mark="circle", radii="2",color=205)
           imedit("sex_frame.fits","e0.fits",cursor="small_objects",value=1,radius=r_small,command="")
#	imcopy ( "sex_frame.fits","e0.fits" )
	   flprcache
           imcopy ("e0.fits", "e0.pl")
}          

           if (access ("medium_objects")) {
           print ("medium objects")

            tvmark (1, coords="medium_objects", mark="circle", radii="5",color=204)
 imedit("p1.fits","e1.fits",cursor="medium_objects",value=1,radius=r_medium,command="")
	   flprcache
           imcopy ("e1.fits", "e1.pl")
}          

           if (access ("large_objects")) {
           print ("large objects")

            tvmark (1, coords="large_objects", mark="circle", radii="10",color=206)
           imedit("p2.fits","e2.fits",cursor="large_objects",value=1,radius=r_large,command="")
	   flprcache
           imcopy ("e2.fits", "e2.pl")
}          
           if (access ("very_large_objects")) {
           print ("very large objects")

            tvmark (1, coords="very_large_objects", mark="circle", radii="30",color=207)
	    imedit("p3.fits","e3.fits",cursor="very_large_objects",value=1,radius=r_vlarge,command="")
	    flprcache
           imcopy ("e3.fits", "e3.pl")
}         
           if (access ("absurdly_large_objects")) {
           print ("absurdly large objects")

            tvmark (1, coords="absurdly_large_objects", mark="circle", radii="45",color=208)
	    imedit("p4.fits","e4.fits",cursor="absurdly_large_objects",value=1,radius=r_alarge,command="")
	    flprcache
           imcopy ("e4.fits", "e4.pl")
}        

# add up the size sorted object masks
 imcopy ("p0.fits", "e99.pl")
 if (access ("e0.pl")) imarith ("e99.pl","+","e0.pl","e99.pl",pixtype="int",calctype="int")
 if (access ("e1.pl")) imarith ("e99.pl","+","e1.pl","e99.pl",pixtype="int",calctype="int")
 if (access ("e2.pl")) imarith ("e99.pl","+","e2.pl","e99.pl",pixtype="int",calctype="int")
 if (access ("e3.pl")) imarith ("e99.pl","+","e3.pl","e99.pl",pixtype="int",calctype="int")
 if (access ("e4.pl")) imarith ("e99.pl","+","e4.pl","e99.pl",pixtype="int",calctype="int")
# display ("e99.pl", 1,zs-, zr-, z1=-1, z2=1,xmag=1, ymag=1)


 imcopy ("e99.pl", "mask_isx_"//rootnum//".pl")

#
#
#

           pclear ("small_objects")
           pclear ("medium_objects")
           pclear ("large_objects")
           pclear ("very_large_objects")
           pclear ("absurdly_large_objects")

pclear ("p0.fits") ; pclear ("p1.fits") ; pclear ("p2.fits") ; pclear ("p3.fits") ; pclear ("p4.fits")
pclear ("e0.fits") ; pclear ("e1.fits") ; pclear ("e2.fits") ; pclear ("e3.fits") ; pclear ("e4.fits")
pclear ("e0.pl")   ; pclear ("e1.pl")   ; pclear ("e2.pl")   ; pclear ("e3.pl")   ; pclear ("e4.pl") 

pclear ("e99.pl") ; pclear ("sex_frame.fits") ; pclear("sex.list") ; pclear("sex.cat")
pclear ("objects_xy.dat")

           old_name = dither_name

}  # close else

# close main while loop
}
   
# clean up
print ("Cleaning up intermediate files... ")
 
end
