
procedure p_quick (n1, n2)

###########################################################
#						          #
#  Written by P McCarthy 10/2004                          #
#                                                         #
#  Makes a quick sky subtracted stack of images           #
#  from frames n1 through n2, usually a dither sequence   # 
#  or set of continguous dither sequences. The frames     # 
#  are first loop combined and then a median sky is       # 
#  formed. This sky is then scaled to each frame          #
#  and subtracted. The first image in the sequence        # 
#  is displayed and the user is asked to mark one or      #
#  more stars. The telescope offsets are then used as     #
#  a first guess for the positions which are improved     #
#  with a two-pass centroiding. An image stack is formed  #
#  using a simple band pixel mask. Calls pclear.cl.       #
#  Uses "center" from digiphot/apphot and "parkey"        #
#  and "keypar" from STSDAS. Report problems to           #
#  pmc2@ociw.edu                                          #
#                                                         #
#                                                         #
###########################################################

struct	  *list1, *list2, *lista
int       n1, n2

begin

# Declarations
  file		offs.dat
  file		afile, brlist
  string	filter_name, type, filename, somename
  string        root_name, filter_old, file_root
  string        name, anothername, root, first_file, first_root
  string	skycount, wcfile, zptfile, rootname, prename
  string        forename, filelist
  real	        ra, dec, dec_old, ra_old, mean_sky, smean
  real          ra_dif, dec_dif, exp_dif, xstart, ystart
  real          newxinit, newyinit, xsum, ysum
  real          newxoff, newyoff, scale, obj_mode, sky_mode
  real          rasec, decsec, xrasec, ydecsec
  real          exposure, exposure_old, first_asec, first_dsec
  real          x, y, x1, y1, x2, y2, x3, y3, rotator
  int           i, k, ra_dif2, dec_dif2, wcnum

  set stdimage = "imt1024"

# Initialize variables 

	pclear ("sky.fits")
	pclear ("sky_list")
	pclear ("skycount")
	pclear ("wcfile")
        first_asec = 0.0
        first_dsec = 0.0
        zptfile = ""
        i = 0

        xstart = 0.0
        ystart = 0.0

# First - make list of frames 
 
             pclear ("loop_list")

            for ( i = n1 ; i <= n2 ; i += 1) {
               if (access ("irp_0000"//i//"_001.fits")) {
                  print ("irp_0000"//i//"_001.fits",>>"loop_list")
               }
 
             if (access ("irp_000"//i//"_001.fits")) {
                  print ("irp_000"//i//"_001.fits",>>"loop_list")
               }

            if (access ("irp_00"//i//"_001.fits")) {
                  print ("irp_00"//i//"_001.fits",>>"loop_list")
               }

            if (access ("irp_0"//i//"_001.fits")) {
                  print ("irp_0"//i//"_001.fits",>>"loop_list")
               }
 
# close for loop 
               }

#   Now combine loops - only 9 loop per pointing combined

              pclear ("combined_list")
              list1 = ""; list1="loop_list"
              while (fscan (list1,filename) != EOF)  {
                 forename = substr(filename,1,9)
                 root = substr(filename,5,9)

                 pclear ("list_1")
                 for (i=1 ; i <= 9 ; i += 1) {

                if (access(forename//"_00"//i//".fits")) {
                print (forename//"_00"//i//".fits",>>"list_1")
                }
            }
             if (access("icx_"//root//".fits")) {}
             else {
             imcombine ("@list_1",output="icx_"//root//".fits",combine="average",scale="none")
             }
             print ("icx_"//root//".fits",>>"combined_list")
              
          }

             filelist = "combined_list"

# Make sky image
             pclear ("sky.fits")
             imcombine ("@"//filelist,"sky.fits", combine="median", scale="mode",masktyp="none")

# Subtract sky image
             list2 = "" ; list2 = filelist
             while (fscan(list2,somename) != EOF) {
             root_name = substr(somename,5,9)
             if (access("isx_"//root_name//".fits")) {}
              else {
   
# Find scaling from sky to object

             pclear ("smode")
             pclear ("skymode")
             pclear ("omode")
             pclear ("objmode")

# Find sky median, object median

             imstat ("sky.fits[20:1000,20:1000]",field="midpt",>"smode")
             imstat ("icx_"//root_name//".fits[20:1000,20:1000]",field="midpt",>"omode")
             
             ! egrep -v '#' smode > skymode
             ! egrep -v '#' omode > objmode
             
             # read values into variables
              lista = "" ; lista = "skymode"
                    while (fscan (lista,sky_mode) !=EOF) {
             
                    }  # close while
             
              lista = "" ; lista = "objmode"
                    while (fscan (lista,obj_mode) !=EOF) {
             
                    }  # close while
             
             # scaling sky to object mode
             
                    scale = (obj_mode / sky_mode)
                    print ("scaling ratio =",scale, "frame = ", root_name)

               imar ("sky.fits","*",scale,"sky.fits") 
               imar ("icx_"//root_name,"-","sky.fits","isx_"//root_name)

# Find and subtract any residual sky level

             pclear ("mean")
             imstat ("isx_"//root_name//".fits",field="midpt",lower=-7500,upper=7500,format=no,>"mean")
             lista= ""; lista="mean"
             while (fscan (lista,smean)!=EOF) {
             mean_sky = smean }
             imarith ("isx_"//root_name//".fits","-",mean_sky,"isx_"//root_name//".fits")
             hedit ("isx_"//root_name//".fits","BPM","/home/panic/iraf/panic/PANIC_mask.pl",add+,verify-)
             } # close if access  
             } # close while

# Now start the dithering process

       i = 0
       list1 = ""; list1="combined_list"
       while (fscan (list1,filename) != EOF)  {

# Find the first position in each dither sequence 

       keypar (filename, "ROTANG", silent+)
       rotator = real(keypar.value)

# Check rotator angle     

       if ( rotator >= 220 && rotator <= 230) { 
       keypar (filename, "asecs", silent+)
       xstart = real(keypar.value)
       keypar (filename, "dsecs", silent+)
       ystart = real(keypar.value)
       }

       if ( rotator >= 40 && rotator <= 50) {
       keypar (filename, "asecs", silent+)
       xstart = real(keypar.value)
       keypar (filename, "dsecs", silent+)
       ystart = real(keypar.value)
       }

       i = i + 1

 
       if (i == 1) 
	   {
# Populate REFERENC file variables

	    first_asec = xstart
	    first_dsec = ystart

           zptfile=filename
           pclear ("init.dat")
           pclear ("slist")

           rootname = substr(filename,1,9)
           prename = substr(filename,5,9)

           display ("isx_"//prename, 1, erase+, bor+, fill+, zs+, xmag=1.0, ymag=1.0)

# Use imexam to interactively get approx coors for fiducial star - mark with "a" then quit

                print ( "")
                print ( "  >>>>> Mark some stars ( a  q )  " )
                print ( "")

           imexamine ( "isx_"//prename,1,keeplog+,logfile='slist' )
           !egrep -v '#' slist | awk '{print $1, $2 }' > init.dat


# Feed imexam co-ors to center task
# Use center task to get refined co-ors of fiducial star;

# Make sure to set centerpars and datapars used by center task as follows:
  center.verify = no
  centerpars.calgori="centroid"
  centerpars.cbox=2.5
  centerpars.cthresh=0
  centerpars.minsnra=1
  centerpars.cmaxite=10
  centerpars.maxshif=1
  centerpars.clean=no
  centerpars.rclean=1
  centerpars.rclip=2
  centerpars.kclean=3
  centerpars.mkcente=no
  datapars.scale=0.125
  datapars.fwhmpsf=2.5
  datapars.emissio=yes
  datapars.sigma=INDEF
  datapars.datamin=INDEF
  datapars.datamax=INDEF
  datapars.noise="poisson"
  datapars.ccdread=""
  datapars.gain=""
  datapars.readnoi=0
  datapars.epadu=1
  datapars.exposur=""
  datapars.airmass=""
  datapars.filter=""
  datapars.obstime=""
  datapars.itime=1

            pclear ("results.dat")

	   center ("isx_"//prename, coords="init.dat", output="results.dat", interactive=no, radplots=no, verify=no, update=no) 

	   !egrep -v '#' results.dat > log.dat
	   !egrep -v 'isx' log.dat >xy.dat
	   !awk '{print $1, $2}' xy.dat > xycoor1.dat
	   list2 = "xycoor1.dat"

# x1, x2 are coors of fiducial star in zeropt ref frame

	   while (fscan (list2, x1, y1) !=EOF) { }

#	   write offsets and ref file name to header of ref file
           prename = substr(filename,5,9)
	   root = substr(filename, 1,9)
	   parkey (0, "isx_"//prename, "XOFF", add+)

	   parkey (0, "isx_"//prename, "YOFF", add+)

	   parkey (zptfile, "isx_"//prename, "REFERENC", add+)

           pclear ("results.dat")
       	   pclear ("xy.dat")
           pclear ("log.dat")
 	   pclear ("init.dat")

	   }   #closing brace for if i==1 loop 
	      
# For files after first, calc coords for center by adding offset to coordinates x1, y1
        
	else if (i > 1)
	   {


	   #read offset values from header
	   keypar (filename, "asecs", silent+)
	   rasec = real(keypar.value)
	   keypar (filename, "dsecs", silent+)
	   decsec = real(keypar.value)
	   
# Calc offset w/r/t ref frame location, find newest coors for fid star

# Need to fix up putting in xstart and ystart variable
           xrasec = real (rasec)
           ydecsec = real (decsec)
	   
# read ref file coors from center output
           list2 = "" ; list2 = "xycoor1.dat"
           while (fscan (list2, x1, y1) !=EOF) {

# calculate new estimated coors in pixels
# xrasec - first_asec is delta in asecs  Note: PANIC scale is 0.126 per SEP
   
           if ( rotator >= 220 && rotator <= 230) { 
           
	   newxinit = x1 + (xrasec -  first_asec) / -0.126
	   newyinit = y1 - (ydecsec - first_dsec) / 0.126

	   print (newxinit, " ", newyinit ,>> "newinits.dat")
              }
           if ( rotator >= 40 && rotator <= 50) { 
           newxinit = x1 - (xrasec -  first_asec) / -0.126
           newyinit = y1 + (ydecsec - first_dsec) / 0.126

           print (newxinit, " ", newyinit ,>> "newinits.dat")
           }

           }
# display image file

           rootname = substr(filename,1,9)
           prename = substr(filename,5,9)

           display ("isx_"//prename, 1, fill+, zs+, xmag=0.5, ymag=0.5)
           tvmark (1, coords="newinits.dat", mark="circle", radii="15",color=206)

# run center 2 times- 1)for general location, 2)to refine using smaller search region
           centerpars.cbox = 5.

	   center ("isx_"//prename, coords="newinits.dat", output="results.dat", interactive=no, radplots=no, verify=no, update=no) 

	   !egrep -v '#' results.dat > log.dat
           !egrep -v 'isx' log.dat >xy.dat
	   !awk '{print $1, $2}' xy.dat > xycoor.dat

	   # set smaller search region
           centerpars.cbox = 2.5

            pclear("results.dat")
            pclear("xy.dat")
            pclear("log.dat")

           center ("isx_"//prename, coords="xycoor.dat", output="results.dat", interactive=no)

           pclear("xycoor2.dat")

           !egrep -v '#' results.dat > log.dat
           !egrep -v 'isx' log.dat >xy.dat
           !awk '{print $1, $2}' xy.dat > xycoor2.dat

# xycoor.dat contains center task's final coors for star
# show center task's coors on display
           tvmark (1, coords="xycoor2.dat", mark="circle",color=206, lengths="15")

!          paste xycoor1.dat xycoor2.dat >xycoor3.dat
# subtract new coors from ref file coors to get offset for each fiducial star
           pclear ("xycoor4.dat")
           
!          awk '{print ($1-$3), ($2-$4)}' xycoor3.dat > xycoor4.dat


# offset = zeropt frame location - current frame location
# calc average offset for all fiducials; xycoor4.dat is offsets list

           k = 0.0
           xsum = 0.
           ysum = 0.
           list2 = "" ; list2 = "xycoor4.dat"
	   while (fscan (list2, x2, y2) !=EOF) { 
           k = k + 1.0
           xsum = xsum + x2
           ysum = ysum + y2
           }
           
           x3 = xsum / k
           y3 = ysum / k

#          print ("x3 =", x3," y3 = ",y3," k = ",k, "file =","isx_"//prename)

           pclear("init.dat")
           pclear("xycoor.dat")
           pclear("newinits.dat")
           pclear("results.dat")
 
     
	   
# write offset (in pix) and name of zerpoint REFERENC file into header 
           prename = substr(filename,5,9)
           root = substr(filename,1,9)

 	   parkey (x3, "isx_"//prename, "XOFF", add+)

 	   parkey (y3, "isx_"//prename, "YOFF", add+)

	   parkey (zptfile, "isx_"//prename, "REFERENC", add+)

	   }	  # close i>1 loop

	}	#close while loop used by fscan; fscan goes on to next file in list


  pclear("init.dat")
  pclear ("slist")
  pclear ("xycoor.dat")
  pclear ("newinits.dat")
  pclear ("results.dat")
  pclear ("xy.dat")
  pclear ("log.dat")

# final cleanup of temp files

  delete ("xycoor1.dat")
  delete ("xycoor2.dat")
  delete ("xycoor3.dat")
  delete ("xycoor4.dat")

            pclear ("off.dat")
            pclear ("obj_list")
            pclear ("stack_"//n1//"_"//n2//".fits")
        

         list1 = ""; list1="combined_list"
         while (fscan (list1,filename) != EOF) {
            prename = substr(filename,5,9)             
            keypar ("isx_"//prename, "xoff")
            newxoff = (real(keypar.value))
            keypar ("isx_"//prename, "yoff")
            newyoff = (real(keypar.value))
            print (newxoff, newyoff ,>> "off.dat")
            print ("isx_"//prename//".fits",>>"obj_list")

             } 

       imcombine ("@obj_list",output="stack_"//n1//"_"//n2//".fits",combine="average",scale="none",offsets="off.dat",masktype="goodvalue",maskvalue=0)

      display  ("stack_"//n1//"_"//n2//".fits",1,zs+)
end

