procedure p_sky1 ( pre, p1, p2 )

string pre
int p1, p2
struct *list1

#   First-pass sky creation and subtraction
#   for PANIC or RetroCam frames
#   
#   Assumes that 'idf' frames exist for normal data 
#   reduction.  These are 
#   linearized, loop-combined, and flattened versions
#   of the raw 'irp' or 'irr' frames.
#
#   However, for quick-looks 'pre' can be 'icp'.
#
#   This produces 'isx' versions with median sky value = 0.0
#
#   Assumes that all frames p1 - p2 have the same
#   filter and exposure time.
#   
#   Assumes that the median value of each frame has 
#   been stored in the header as 'FRMEDIAN"

begin
   int i
   real sky_med, frm_med, ratio
   string pre_frame, isx_frame, root_name, sp1

   p_pre_list ( pre, p1, p2 )

#  Combine images using median and save sky frame

   padindex(p1) ; sp1 = padindex.output
   pclear ("sky.fits") ; pclear ( "sky_"//sp1 )

   imcombine ("@"//pre//"_list","sky.fits", combine="median", scale="mode",masktyp="none")
   p_store ( "sky" )
   imcopy ( "sky", "sky_"//sp1 )

   keypar("sky", "FRMEDIAN",silent+)
   sky_med = real(keypar.value)

#   Scale the sky frame to the median value of each frame and subtract.
#   Recalculate sky every time through.

   pclear ( "isx_list" )
   list1 = "" ; list1 = pre//"_list"
   while ( fscan ( list1, pre_frame ) != EOF )    {

      p_store ( pre_frame )
      keypar(pre_frame, "FRMEDIAN",silent+)
      frm_med = real(keypar.value)
      ratio = frm_med / sky_med

      root_name = substr(pre_frame,5,9)
      isx_frame = "isx_"//root_name//".fits"
      pclear (isx_frame )

      pclear ( "dummy" )
      imar ( "sky.fits", "*", ratio, "dummy" )
      imar ( pre_frame, "-", "dummy", isx_frame )
      print ( isx_frame, >> "isx_list" )

#   Remove residual sky treated as a constant (the median) for now.

      p_offset_level ( isx_frame, "median" )

#   Set frame edges to zero

      p_zero_edges ( isx_frame )
   }

#   p_movie ( "isx", p1, p2 )

end
