procedure p_sky2 ( p1, p2, p3, p4 )

int p1, p2, p3, p4
struct *list1

#	Alternative sky subtraction method for PANIC or RetroCam frames.
#
#	In this case the previously created sky frame (in p_skysub.cl)
#	 'sky_000p3_000p4' must exist. Scaled versions
#	of it are subtracted from the p1,p2 'idf' frames to produce
#	p1,p2 'isx' frames. The p1,p2 'idf' frames are not used at
#	all in creating the sky.
#
#	USES:
#
#	1.   Very busy frame with too many stars to clean away.
#	2.   A frame contining a large galaxy that cannot be medianed away.
#	3.   A sequence that has too few frames to median to get a decent sky.
#	     For example standard stars, for which single stacked images are 
#	     wanted.
#
#	CAVEATS:
#
#	This will not work well if the airmasses are too diffeerent or if 
#	>~ 1/2 hour has elapsed between sequences. 
#	
#	Assumes that 'idf' frames exist. These are 
#	linearized, loop-combined, and flattened versions
#	of the raw 'irp' or 'irr' frames.
#
#	This produces 'isx' versions with median sky value = 0.0
#
#	Assumes that all frames p1 - p2 have the same
#	filter and exposure time.
#	
#	Assumes that the median value of each frame has 
#	been stored in the header as 'FRMEDIAN"

begin
	int i
	real sky_med, frm_med, ratio
	string idf_frame, isx_frame, root_name
	string onumber, onumber3, onumber4
	string sky, date

	padindex(p3) ; onumber3 = padindex.output
	padindex(p4) ; onumber4 = padindex.output

	sky = "sky_"//onumber3//"_"//onumber4//".fits"

	keypar(sky,"NIGHT",silent+)
	date= str(keypar.value)

	if ( access ( sky ))	{

	p_pre_list ( "idf", p1, p2 )

#	Combine images using median

	p_store ( sky )

	keypar ( sky, "FRMEDIAN", silent+ )
	sky_med = real(keypar.value)

#	Scale the sky frame to the median value of each frame and subtract.
#	Recalculate sky every time through.

	pclear ( "isx_list" )
	list1 = "" ; list1 = "idf_list"
	while ( fscan ( list1, idf_frame ) != EOF ) 	{

	  keypar(idf_frame, "FRMEDIAN",silent+)
	  frm_med = real(keypar.value)
	  ratio = frm_med / sky_med


##########  CHECK THIS !! Commented out 11sep2004

#       parkey ("mask_"//date//".pl", idf_frame, "BPM", add+)

             root_name = substr(idf_frame,5,9)
	     isx_frame = "isx_"//root_name//".fits"
             pclear (isx_frame )

	pclear ( "dummy" )
#  CRB:  fixed the following.  It was using "sky.fits" instead of sky variable.
#   Probably cruft left over from original p_sky
	imar (sky, "*", ratio, "dummy" )
	imar ( idf_frame, "-", "dummy", isx_frame )
	print ( isx_frame, >> "isx_list" )

#	Remove residual sky treated as a constant (the median) for now.

	p_offset_level ( isx_frame, "median" )

#	Set frame edges to zero

	p_zero_edges ( isx_frame )
												}

	p_movie ( "isx", p1, p2 )

	}	else	{
	psorry ( sky )
	}
end
