procedure p_skysub (  p1, p2 )

########################################################
#  Written by P McCarthy and J Wilson, 8/19/99         #
#  This  task imcombines files to create a sky image   #
#  (using a combined object and bad pixel mask),       #
#  which is scaled by the object_mode/sky_mode ratio,  #
#  and is subtraced from the idf file to create an     #
#  is2 file. Output files are is2*fits, iss*fits and   #
#  mask2*pl files.                   #
########################################################
#
#   Revised 09mar2003:  SEP
#   Revised 13aug2004:  SEP
#
#   Take p1,p2 style input, get date out of header of first idf frame

int p1, p2
struct     *list1, *list2, *lista
 
#string     date      {prompt="Date in 14May1999 format"}
#file     filelist   {prompt="List of isx*c1.fits input files"}

begin

# Declarations
   file      afile
   file          filelist
   string   outname, somefile, skyname, mask_file, filtname
   string        root_name, root_sky, dither_name, dither_root
   string        new_mask, root, anothername
   string        type, date, pp1, pp2
   int           n1, n2, j, xshift, yshift, xs, ys
   real          sky_mode, obj_mode, scale, exp_time, zpt
   real          sky_bright, sky_rat
   string        orig, test
   
   # set stdimage = "imt1024"
   
   # Make sure variables are clean
   pclear ("top")
   pclear ("afile")
   pclear ("sky_list2")
   pclear ("sky_mean")
   pclear ("obj_mean")
   pclear ("sky_list")
   pclear ("is2*.fits")
   sky_mode = 0
   obj_mode = 0
   scale = 1
   root_sky = ""

  #   Make isx list

   pclear ( "isx_list" )
   p_pre_list ( "isx", p1, p2 )
   filelist = "isx_list"

   lista = "" ; lista = filelist
   while (fscan (lista,somefile) !=EOF) {
      #  find dither and root names and reset offsets

      keypar (somefile, "referenc")
      dither_name = str(keypar.value)
      dither_root = substr(dither_name,1,9)
      root_name = substr(somefile,5,9)

      keypar (somefile, "xoff2")
      xshift = real(keypar.value)
      xs = xshift * -1

      keypar (somefile, "yoff2")
      yshift = real(keypar.value)
      ys = yshift * -1

      keypar (somefile, "NIGHT")
      date = str(keypar.value)

      mask_file = "mask_"//dither_root//".pl"

      # copy mask file (mask_isx) into mask template to create mask of 
      # correct size (mask2) and update header in sky frames
      # Skip all this if the mask2 files exist

      if (access ("mask2_"//root_name//".pl")) { 
           parkey("mask2_"//root_name//".pl", "idf_"//root_name//".fits", "BPM",add+)
      } else {
         pclear ("mask2_"//root_name//".pl" )
         imshift (mask_file, "mask2_"//root_name//".pl", xs, ys, constan=1, boundar="constant")
         imcopy ("mask2_"//root_name//".pl"//"[1:1024,1:1024]", "mask2_"//root_name//".pl")
         imarith ("mask2_"//root_name//".pl", "+", "mask_"//date//".pl", "mask2_"//root_name//".pl")
         imreplace ("mask2_"//root_name//".pl", lower=1.01, upper=INDEF, value=1)
         parkey("mask2_"//root_name//".pl", "idf_"//root_name//".fits", "BPM",add+)
         flprcache

      }   # close else for access check
   }   # close while over files in input list - all masks are now done

   #   Before doing skies for each 'idf' frame, compute the mean sky for ALL
   #   idf frames in this sequence
   #   Revised 10mar2003    SEP

   p_pre_list ( "idf", p1, p2 )
 
   padindex(p1) ; pp1 = padindex.output
   padindex(p2) ; pp2 = padindex.output
 
   # combine idf frames to make sky image of all frames in the set
 
   if (! access("sky_"//pp1//"_"//pp2//".fits" ) ) {
      pclear ( "sky_"//pp1//"_"//pp2//".fits" )
 
      imcombine("@"//"idf_list","sky_"//pp1//"_"//pp2//".fits",combine="median",scale="mode",masktype="goodvalue",maskvalue=0.,blank=-999, weight="none")
 
      # fix any blank pixels in the sky frame
      p_store ( "sky_"//pp1//"_"//pp2 )
      keypar ( "sky_"//pp1//"_"//pp2, "FRMEDIAN", silent+ )
      sky_mode = real(keypar.value)
 
      imreplace( "sky_"//pp1//"_"//pp2, value=sky_mode,lower=INDEF,upper=-500)
      p_edges ( "sky_"//pp1//"_"//pp2 )
   }

   # Find surrounding images for sky frame; use all but the two at the same 
   # dither position. 

   list1 = "" ; list1 = filelist
   while (fscan (list1,somefile) !=EOF) {
      root_name = substr(somefile,5,9)
      print ( somefile )

      keypar(somefile,"FILE") ; orig=str(keypar.value)
      
      #  make list of sky frames, exluding "somefile", change to idf frames;
      #  i.e., make sure file to sky subtract is not in sky_list. Here we are 
      #  checking the original numbering of files. If "FILE" is the same as the
      #  one being tested, it is not included.
      
      list2 = "" ; list2 = filelist
      while (fscan (list2,skyname) !=EOF) {
      
         # print ("")
         # print ( skyname )
         # print ("")
      
         keypar(skyname,"FILE") ; test=str(keypar.value)
         root_sky = substr(skyname,5,9)
      
         # 09jun2005 - delete test on FILE ; IRAF seems to be corrupting; SEP & PLW 
      
         #  if (skyname != somefile && test != orig ) print ("idf_"//root_sky//".fits", >> "sky_list2")
         if (skyname != somefile ) print ("idf_"//root_sky//".fits", >> "sky_list2")
      }  # close sky_list while
      
      tail ("sky_list2", nlines=20)
      
      pclear("sky_"//root_name//".fits")
      pclear("is2_"//root_name//".fits")
      
      # combine frames to make sky image
      imcombine("@"//"sky_list2","sky_"//root_name//".fits",combine="median",scale="median",masktype="goodvalue",maskvalue=0.,blank=-999, weight="none", statsec="[41:984,41:984]")
      pclear ("smode")
      pclear ("skymode")
      pclear ("omode")
      pclear ("objmode")
      
      # find sky mode, object mode
      #
      # Revised 10mar2003   SEP - Use p_store to get better mode or 
      #   median values and try median
      
      p_store ( "sky_"//root_name )
      
      keypar ( "sky_"//root_name, "FRMEDIAN", silent+ )
      sky_mode = real(keypar.value)
      
      keypar ( "idf_"//root_name, "FRMEDIAN", silent+ )
      obj_mode = real(keypar.value)
      
      # imstat ("sky_"//root_name//".fits",field="mode",lower=0,>"smode")
      #imstat ("idf_"//root_name//".fits",field="mode",lower=0,>"omode")
      
      #! egrep -v '#' smode > skymode
      #! egrep -v '#' omode > objmode
      #
      #read values into variables
      #list2 = "" ; list2 = "skymode"
      #while (fscan (list2,sky_mode) !=EOF) {
      #}  # close while
      #   
      #list2 = "" ; list2 = "objmode"
      #while (fscan (list2,obj_mode) !=EOF) {
      #   
      #}  # close while
      
      # scaling sky to object mode
      
      scale = (obj_mode / sky_mode)
      
      # computing sky surface brightness
      
      keypar("idf_"//root_name//".fits","EXPTIME")
      exp_time = real(keypar.value)
      keypar("idf_"//root_name//".fits","ZMAG")
      zpt = real(keypar.value)
      
      sky_rat = (obj_mode/exp_time)
      if (sky_rat >= 0) { 
         sky_bright = -2.5 * log10(sky_rat) + zpt - 3.495
         print ("idf_"//root_name," ","sky brightness ="," ",sky_bright," Counts/sec/pix ="," ",sky_rat,>>"sky_log") 
         print ("sky scaling factor ="," ", scale," ","sky brightness ="," ",sky_bright)
      } else  { 
         print ("negative sky value")
      }
      # scale sky by object mode/sky mode ratio
      
      pclear ( "osky_"//root_name//".fits")
      imcopy("sky_"//root_name//".fits","osky_"//root_name//".fits")
      imarith("sky_"//root_name//".fits","*",scale,"sky_"//root_name//".fits",pixtype="real",calctyp="real")
      
      # fix any blank pixels in the sky frame
      
      imreplace("sky_"//root_name//".fits",value=obj_mode,lower=INDEF,upper=-500)
      
      
      # subtract the sky frame from the flattened frame
      imar("idf_"//root_name//".fits","-","sky_"//root_name//".fits","is2_"//root_name//".fits",pixtype="real",calctyp="real")
      flprcache
      
      #
      #  Revised 10mar2003 SEP  -  zero out the edges of the is2 frame
      #
      p_offset_level ( "is2_"//root_name , "median" )
      p_zero_edges ( "is2_"//root_name )
      p_store ( "is2_"//root_name )
      
      display ("sky_"//root_name//".fits",1)
      display ("is2_"//root_name//".fits",1,zs-,zr-,z1=-200,z2=200)
      
      #  imstat ("sky_"//root_name//".fits",field="image,midpt")
      #  imstat ("is2_"//root_name//".fits",field="image,midpt")
      
      # imdelete ("sky_"//root_name//".fits",ver-)
      
      pclear("sky_list2") 
      pclear("top")
      pclear("sky_list") 

   # close main while 
   }

   # put correct mask back into idf and is2 file headers

   lista = "" ; lista = filelist
   while (fscan (lista,somefile) !=EOF) {
      root_name = substr(somefile,5,9)

      keypar ("idf_"//root_name//".fits", "filter2")
      filtname = str(keypar.value)
      parkey ("mask_"//date//".pl", "is2_"//root_name//".fits", "BPM")
      parkey ("mask_"//date//".pl", "idf_"//root_name//".fits", "BPM")

   }   # close while

   # clean up
   # pclear ("sky_"//root_name//".fits",ver-)
   pclear ("top")
   pclear ("sky_list")
   # pclear ("sky_list2")
   pclear ("smode")
   pclear ("omode")
   #delete ("sky_log")
   pclear  ("skymode")
   pclear ("objmode")
end
