procedure p_stack (p1, p2, sname)

int p1, p2
string    sname         {prompt="Root name for stacks"}
struct   *list1, *list2

##############################################################
#  Written by P McCarthy and J Wilson, 8/18/99               #
# Modified 9/5/02                                            # 
# For each dither sequence, finds offset of each frame w/r/t #
# the REFERENC frame by comparing frame's coordinates for a  #
# user selected group of stars to the coordinates for those  #
# stars in the REFERENC frame. Writes offset values (in      #
# pixels) into the header of the is2 and idf files as XOFF2  #
# and YOFF2, as well as the REFERENC frame  (keyword         #
# REFERENC).                                                 #
# Input is a list of the is2*.fits files.                    #
# NOTE: Must start display tool before running this task     #
#                                                            #
##############################################################
#
#        Revised 10mar2004   SEP.
#          Now takes p1,p2 input and created 'is2_list'
        
begin
   string    filelist, filename, fname, onumber
   string    root_name, anothername, type
   string    astro_list, root, keep, update, stack
   file      offsets.dat
   real      xshift, yshift, xoff, yoff, rax, decx, xs, ys, x, y
   real      xdif, ydif, x1, y1, x2, y2, xfid, yfid, xfid_off2, yfid_off2
   real      psf_dif
   int       m 

   # initialize variables and recreate list

   pclear ("offsets.dat") ; pclear ("flist") ; pclear ("imcen") 
   pclear ("sex.fits") ; pclear ( "fid.dat")
   set stdimage=imt1024

   p_pre_list ( "isd", p1, p2 )
   filelist = "isd_list"

   # Get fiducial XY from header of first frame

   padindex(p1)
   onumber = padindex.output
   fname = "idf_"//onumber

   keypar (fname//".fits", "XFID", silent+)
   xfid = real(keypar.value)
   keypar (fname//".fits", "YFID", silent+)
   yfid = real(keypar.value)
   keypar ("is2_"//onumber//".fits", "XOFF2", silent+)
   xshift = real(keypar.value)
   keypar ("is2_"//onumber//".fits", "YOFF2", silent+)
   yshift = real(keypar.value)

   xfid_off2 = xfid + xshift ; yfid_off2 = yfid + yshift
   pclear ( "xy.in" ) ; print (xfid_off2," ",yfid_off2,>>"xy.in")


   # loop through files and get offsets, write to file
   list1 = ""; list1=filelist
   while (fscan (list1,filename) != EOF)  {

   # find the first position in each dither sequence 

      root_name = substr(filename,1,9)

      keypar (root_name//".fits", "XOFF2", silent+)
      xshift = real(keypar.value)
      keypar (root_name//".fits", "YOFF2", silent+)
      yshift = real(keypar.value)

      print (root_name//".fits",>>"flist")
      print (xshift," ",yshift,>>"offsets.dat")
   }
   pclear("stack_"//sname//".fits") ; pclear ( "psf.out" ) ; pclear ( "psf.2" )
   # sigma-map added by CRB:
   pclear("stack_"//sname//"_sigma1.fits")
   imcombine("@flist","stack_"//sname//".fits",offsets="offsets.dat", combine="average", masktyp="badvalue", maskval=1., scale="none", sigma="stack_"//sname//"_sigma1.fits")

   display("stack_"//sname//".fits",1,fill+,zs+,erase-,bor+,zs-,zr-, z1=-500,z2=500)

   pclear ( "psf.out" ) ;  pclear ( "psf.3" )
   imexamine("stack_"//sname//".fits",1,keeplog+,logfile="psf.out", imagecur="xy.in")

   !/usr/bin/tail -1 psf.out | awk '{ print $13, $15, ($13+$15)/2. }' > psf.2

   imrename ("stack_"//sname//".fits","sex.fits")

   ! sex sex.fits -c default_is2.sex 
   # CRB (3/8/2006):  should throw out objects with flag != 0
   ! egrep -v '#' sex.cat | egrep "0$" >sex.list
   ! awk '{print $9, $10}' sex.list >objects.dat

   imrename ("sex.fits","stack_"//sname//".fits")

   pclear("sex.cat")
   pclear("sex.list")
   tvmark (1, coords="objects.dat", mark="circle", radii="10",color=205)

   imcentroid ("@flist", reference="stack_"//sname//".fits", coords="objects.dat",maxshift=20,shifts="offsets.dat",verbose-,>"imcen")


   !grep fits imcen > pp
   !awk '{print $2,$4}' pp >new_offsets.dat
   !paste offsets.dat new_offsets.dat >kk
   !awk '{print $1-$3,$2-$4}' kk >dif
   !head -1 dif >dif2

   #  Rev 30aug2004: here we really want the average
   #
   pclear ("newer_offsets.dat")
   # list2 = " " ; list2 = "dif2"
   # while (fscan (list2,xdif,ydif) != EOF)  { }

   xdif = 0. ; ydif = 0. ; m=0
   list2 = " " ; list2 = "dif"
   while (fscan (list2,x,y) != EOF)  { 
      xdif += x ; ydif += y ; m += 1
   }
   xdif = xdif/m ; ydif = ydif/m

   list2 = " " ; list2 = "new_offsets.dat"
   while (fscan (list2,x1,y1) != EOF)  {
      x2 = x1 + xdif
      y2 = y1 + ydif
      print (x2," ",y2,>>"newer_offsets.dat") 
   }
   pclear("stack_"//sname//"a.fits") ; pclear("stack_"//sname//"b.fits")
   pclear ( "psfa.out" ) ; pclear ( "psfa.2" ) ; pclear ( "psf_tst" )
   # Sigma map added by CRB
   pclear("stack_"//sname//"a_sigma1.fits")
   pclear("stack_"//sname//"b_sigma1.fits")

   imcombine("@flist","stack_"//sname//"a.fits",offsets="newer_offsets.dat", combine="average", masktyp="badvalue", maskval=1., scale="none",  sigma="stack_"//sname//"a_sigma1.fits")

   imcombine("@flist","stack_"//sname//"b.fits",offsets="newer_offsets.dat", combine="average", masktyp="badvalue", maskval=1., scale="none", reject="pclip", pclip=-0.5,lsigma=5.,hsigma=5.,nkeep=5, sigma="stack_"//sname//"b_sigma1.fits")

   display("stack_"//sname//"a.fits",1,fill+,zs+,erase-,bor+,zs-,zr-,z1=-500, z2=500)
   imexamine ("stack_"//sname//"a.fits",1,keeplog+,logfile="psfa.out", imagecur="xy.in")

   !/usr/bin/tail -1 psfa.out | awk '{ print $13, $15, ($13+$15)/2. }' > psfa.2
   !paste psf.2 psfa.2 | awk '{print $6 - $3}' > psf_tst

   list2 = " " ; list2 = "psf_tst"
   while (fscan (list2,psf_dif) != EOF)  { }

   if (psf_dif <= 0.) {

      !paste flist newer_offsets.dat > nlist.dat

      list1 = ""; list1="nlist.dat"
      while (fscan (list1,fname,x, y) != EOF)  {
         xoff = x
         yoff = y

         hedit (fname,"XOFF2",xoff,add+,verify-)
         hedit (fname,"YOFF2",yoff,add+,verify-)

      }
      print ( " Headers updated"  )
   } 
   else {
      print ("") ; print ("") 
      print ( " Headers not changed"  )
      print ("")
   }

   !paste flist newer_offsets.dat > nlist.dat

   pclear("comblist_"//sname)

   list1 = ""; list1="nlist.dat"
   while (fscan (list1,fname,x, y) != EOF)  {
      root = substr(fname,1,9)
      keypar (fname, "RA", silent+)
      rax = real(keypar.value)
      keypar (fname, "DEC", silent+)
      decx = real(keypar.value)
      keypar (fname, "XOFF2", silent+)
      xs = real(keypar.value)
      keypar (fname, "YOFF2", silent+)
      ys = real(keypar.value)
 
      print ("      ",root," ",rax," ",decx," ",xs," ",ys,>>"comblist_"//sname)

   }
end
