procedure p_tflat ( p1, p2 )

int p1, p2
struct *list1

#	Assumes that 'icd' linearized, loop-combined, and 
#	dark-subtracted frames exist. 
#
#	'p1' and 'p2' are the starting and ending 
#	frame numbers (of valid non-saturated) frames of a set  
#	taken as the sky is changing during twilight.
#
#	It is assumed that 'p_chk_headers' and 'p_fix_headers' have 
#	been run and that therefore it is not necessary to check  
#	for filter consistency.
#
#	The existence of frames within a set is not assumed (in
#	case it has been necessary to delete some) but is worked
#	around. However frames p1 and p2 must exist.
#
#	This script assumes that the data have been taken
#	according to the procedure specified for twiflats
#	in the PANIC camera control gui. IN PARTICULAR, the
#	telescope must be moved between frames so that field
#	stars can be medianed away. 
#
#	If there are any bright stars in these frames, this 
#	script will not work well unless N > 10 or even 15.
#

#
#	It is assumed FOR NOW that they all have the same exposure time.
#	These difference frames are imcombined with scaling on.
#
#	
#	dvorak:/home/persson/iraf/panic_scripts/p_tflat.cl
#
#	Rev: 28nov2003  SEP
#	Rev: 12mar2004  SEP
#	Rev: 13mar2004  SEP
#	Rev: 14mar2004  SEP - now wants dark-subtracted frames.

begin
	string onumber1, onumber2, date, filter_p1, filter_p2, default_path

#	Get filters and date out of headers

	padindex(p1) ; onumber1 = padindex.output
	padindex(p2) ; onumber2 = padindex.output

	  keypar("icd_"//onumber1//".fits", "FILTER",silent+)
	  filter_p1 = str(keypar.value)

	  keypar("icd_"//onumber2//".fits", "FILTER",silent+)
	  filter_p2 = str(keypar.value)

	  keypar("icd_"//onumber1//".fits", "NIGHT",silent+)
	  date = str(keypar.value)

	if ( filter_p1 == filter_p2 )	{

	p_pre_list ( "icd", p1, p2 )

	pclear ( "flat" )  ;  pclear ("nptflat"//filter_p1//"_"//date)

	imcombine ( "@"//"icd_list", "flat", combine="median",scale="median",zero="none",reject="pclip", pclip=-0.5,lsigma=3.,hsigma=3.,nkeep=1)

#	Normalize
#	Fix edges, which also stores image statistics

	p_norm ( "flat", "flat", "median" )
	p_edges ( "flat" )

#	Get rid of nonsensical values. SEE COMMENTS IN p_dflat.cl

	imreplace ( "flat", value=1, lower=INDEF, upper=0.6 )
	imreplace ( "flat", value=1, lower=1.5, upper=INDEF )


	pclear ( "nptflat"//filter_p1//"_"//date )
	imrename ( "flat", "nptflat"//filter_p1//"_"//date )

	default_path = "home$panic_files/flats"
	copy ( "nptflat"//filter_p1//"_"//date//".fits", default_path )

						}	else	{
	print ( "" )
	print ( "  >> SORRY: Filters are not consistent " )
	print ( "" )
									}		

end

	  















	  













