procedure p_tst ( p1, p2)

int p1, p2
struct *list1

#	Photometry test program
#	The photometric aperture size is 5 arseconds radius.
#	DEFINITIONS
#	
#	instrumental magnitude: inst_mag = 20.000 - 2.5log10(net ADU)
#
#	the value of 20.000 is set in photpars.zmag
#
#	night constant:  ngt = inst_mag + 1.0857*log(etime) - dmag_air - std_mag
#	
#	magnitude of an object:    mag = zmag - 2.5log10(net ADU)
#	
#	zmag:   zmag = 20.000 + 1.0857*log(etime) - dmag_air - ngt 
#	
#	zpt: the magnitude of an object netting 1 ADU/sec at 1 airmass
#
#	zpt = 20.000 - ngt


begin
	string isx_frame, is2_frame, root_name
	string onumber1, onumber2
	string std, stdname, stdo, filtero, filter, date
	real x, y, x_std, y_std, airmass, a, std_mag, etime, dmag_air
	real inst_mag, test, scale, ngt, zmag, zpt
	int j, jmax, lctime

	padindex(p1) ; onumber1 = padindex.output
	padindex(p2) ; onumber2 = padindex.output

	std = "stk_std_"//p1//"_"//p2
        display (std, 1, fill+, zs+, xmag=0.5,ymag=0.5)

#	Photometry 

#	Get scale, std name, x, y, exptime, airmass, and filter(s) from header

	keypar(std,"SCALE",silent+)
	scale=real(keypar.value)
	datapars.scale = scale

	keypar(std,"NIGHT",silent+)
	date = str(keypar.value)

	keypar(std,"OBJECT",silent+)
	stdname = str(keypar.value)

	keypar(std,"LC-TIME",silent+)
	lctime = int(keypar.value)

	keypar(std,"X_STD",silent+)
	x = real(keypar.value)

	keypar(std,"Y_STD",silent+)
	y = real(keypar.value)

	keypar(std,"EXPTIME",silent+)
	etime = real(keypar.value)

	keypar(std,"AIRMASS",silent+)
	airmass = real(keypar.value)

	keypar(std,"FILTER",silent+)
	filter = str(keypar.value)

	if ( filter == "Yc" ) dmag_air = 0.10*(airmass - 1.0)
	if ( filter == "Jc" ) dmag_air = 0.12*(airmass - 1.0)
	if ( filter == "Hc" ) dmag_air = 0.04*(airmass - 1.0)
	if ( filter == "Ks" ) dmag_air = 0.08*(airmass - 1.0)
																
#	Get std magnitude

        p_get_std_mag (stdname, filter)

	list1 = "" ; list1 = "std_mag"
	while ( fscan ( list1, stdo, filtero, std_mag ) != EOF ) {} 

#	Set up Photometry Parameters

	photpars.aperture = 5.
	photpars.zmag = 20.
	centerpars.cbox = 5.
	datapars.datamin = -2000.
	fitskypars.salgorithm = "median"
	fitskypars.annulus = 7.
	fitskypars.dannulus = 5.
	datapars.scale = scale

        pclear ( "std_inst" ) ; pclear ( "std_ph" ) ; pclear ("stdxy.dat")
        pclear ( std//".mag.1" )

	print ( x,"  ", y, > "stdxy.dat")
	tvmark (1, coords="stdxy.dat", mark="circle", radii="40",color=204)


        phot(std ,interac-,verify-, coords="stdxy.dat" )
        tail  ( std//".mag.1", nlines=1,  >> "std_ph" ) 

	!awk ' {print $5}' std_ph > std_inst

	list1 = "" ; list1 = "std_inst"
	while ( fscan ( list1, inst_mag ) != EOF ) {} 

#	print ("")
#	print (inst_mag,"  ",1.0857*log(etime),"  ",dmag_air,"  ",std_mag )
#	print ("")

	ngt = inst_mag + 1.0857*log(etime) - dmag_air - std_mag
	zpt = 20.000 - ngt

	printf ("%s%5.0f%5.0f%s%8.0f%s%8.3f%8.3f%8.3f%s\n", stdname, p1, p2, "  "//date, lctime, "  "//filter, airmass, ngt, zpt,"  10", >> "ngt_"//date )

end




