#!/usr/bin/env python
from myiraf import *
import pyfits
import sys,os,string,re
import numarray as num
from glob import glob

#  /home/persson/iraf/panic_scripts/p_linearize.cl

#  PANIC (camera = p) or RetroCam (r) linearization script. 
#  Linearizes 'irp' or 'irr' frames n1 through n2, skipping 
#  missing ones, and producing 'izp' or 'izr' frames. Parameters
#  and basic algorithm from Paul Martini.
#  Rev:  30nov2003   SEP
#
#  Set readout time parameters for each camera
#  All times in seconds.
#
#  reads_ep = number of end-point reads
#  pre_ep = number of pre-endpoint reads; set to 1 for all cameras.
#
#  tf = frame readout time
#  td = delay time
#  tfd = tf + td
#
#  ta = acquisition time
#  t_pre_ep = pre-endpoint readout time = pre_ep * tfd 
#  t_read = total effective frame readout time = reads_ep * ( tfd + ta )


def p_lincor_sep (data, p, c1, c2, c3, linthresh):
   #   Do linearity correction for an image
   #
   #   p = a/(a + exptime), where a is the overhead due
   #   to prereads and reads. The output image is
   #   [(1+p)*pic](corr) - [p*pic](corr)
   #
   #  The linearity correction is a polynomial above a 
   #  threshold. 
   p1 = 1.0 + p

   #output = "izp_"//substr(pic,5,25)
   #    pclear ( output )

   data1 = data * p1    # data1 is now type real
   cor = c1 + c2*data1 + c3*num.power(data1,2)
   out1 = num.where(num.less_equal(data1, linthresh), data1, cor)

   data2 = data * p    # data2 is now type real
   cor = c1 + c2*data2 + c3*num.power(data2,2)
   out2 = num.where(num.less_equal(data2, linthresh), data2, cor)

   output = out1 - out2
   # Convert back to original type (by truncation)
   #output = output.astype(data.typecode())
   return output


pre_ep = 1.

camera = sys.argv[1]
n1 = int(sys.argv[2])
n2 = int(sys.argv[3])

if camera.lower() in ['r','retro','retrocam']:
   td = 0.010 ; tf = 2.0 ; ta = 0.085
elif camera.lower() in ['p', 'panic']:
   td = 0.010 ; tf = 2.0 ; ta = 0.085
   c1 = 4.291443e+01 ; c2 = 9.752524e-01 ; c3 = 1.962545e-06 ; linthresh = 8000.


tfd = tf + td
t_pre_ep = pre_ep * tfd 

# Get list of all pictures to be linearized
# Output is "raw_pic_list"

p_raw_list = ["irp_%05d" % i for i in range(n1, n2+1)]
i = 0
step = len(p_raw_list) / 80
for base in p_raw_list:
   sublist = glob(base + '*.fits')
   for pic in sublist:
      f = pyfits.open(pic)
      d = f[0].data
      exptime = f[0].header['EXPTIME']
      readmode = f[0].header['READMODE']
      if readmode == 'Double':  reads_ep = 1.0
      if readmode == 'Quadruple':  reads_ep = 2.0
      if readmode == 'Sextuple':  reads_ep = 3.0
      if readmode == 'Octuple':  reads_ep = 4.0
      f.close()
 
      t_read   = reads_ep * ( tfd + ta )
 
      #  Linearize
 
      delta = ( t_pre_ep + t_read / 2. ) 
      p = delta / ( delta + exptime )       
      print p_lincor_sep (8000, p, c1, c2, c3, linthresh)
      print p_lincor_sep (30000, p, c1, c2, c3, linthresh)
 
      newdata = p_lincor_sep ( d, p, c1, c2, c3, linthresh )
      f[0].data = d
      f.writeto(pic.replace('irp','izp'))
      #qdump(pic.replace('irp','izp'), newdata, pic)
      if i % step == 0:
         print "*",
         sys.stdout.flush()
      

