#!/usr/bin/env python

from pyraf import iraf
from myiraf import *
import os,string,re, sys
from rdarg import rdarg
from ReadSex import readsex
import Numeric as num

##############################################################
#  Written by P McCarthy and J Wilson, 8/18/99		     #
# Modified 9/5/02                                            # 
# For each dither sequence, finds offset of each frame w/r/t #
# the REFERENC frame by comparing frame's coordinates for a  #
# user selected group of stars to the coordinates for those  #
# stars in the REFERENC frame. Writes offset values (in      #
# pixels) into the header of the is2 and idf files as XOFF2  #
# and YOFF2, as well as the REFERENC frame  (keyword	     #
# REFERENC).						     #
# Input is a list of the is2*.fits files.		     #
# NOTE: Must start display tool before running this task     #
#							     #
###############################################################
#
#	Revised 10mar2004   SEP.
#	  Now takes p1,p2 input and created 'is2_list'
#
#  Ported to Pyraf by Chris Burns, 10mar2006
#    also added noise-map output trough the sigmas= imcombine option

if len(sys.argv) < 4:
   print "Usage:  p_stack.py p1 p2 sname"
   sys.exit(1)

p1 = int(sys.argv[1])
p2 = int(sys.argv[2])
sname = sys.argv[3]

for file in ['offsets.dat','flist','imcen','sex.fits','fid.dat']:
   nuke(file)

filelist = ["isd_%05d.fits" % i for i in range(p1,p2+1)]
# Now, check to see if they all exist... if not, delete from the list:
for file in filelist:
   if not os.path.isfile(file):
      del filelist[filelist.index(file)]

# Get fiducial XY from header of first frame
fname = "idf_%05d.fits" % p1
xfid = get_header(fname, "XFID", float)
yfid = get_header(fname, "XFID", float)
xshift = get_header(fname, "XOFF2", float)
yshift = get_header(fname, "XOFF2", float)
xfid_off2 = xfid + xshift ; yfid_off2 = yfid + yshift
f = open('xy.in','w')
print >> f, xfid_off2, yfid_off2
f.close()

f = open('offsets.dat','w')
f2 = open('flist','w')
offsets = [[],[]]
# loop through files and get offsets, write to file
for filename in filelist:
   # find the first position in each dither sequence 

   xshift = get_header(filename, "XOFF2", float)
   yshift = get_header(filename, "YOFF2", float)
   offsets[0].append(xshift)
   offsets[1].append(yshift)
   print >> f2, filename
   print >> f, xshift, yshift

f.close()
f2.close()
offsets = num.array(offsets)

# First attempt at imcombine using offset already in the header.
stack = 'stack'+sname+'.fits'
nuke(stack);  nuke('psf.out');  nuke('psf.2')
iraf.imcombine("@flist",stack,offsets="offsets.dat",combine="average", 
               masktyp="badvalue", maskval=1., scale="none")

iraf.display(stack,1,fill=1,erase=0,border_erase=1,zscale=0,zrange=0,
             z1=-500,z2=500)
nuke('psf.out');  nuke('psf.3')
out = iraf.imexamine(stack,1,imagecur="xy.in", Stdout=1)
psfs = map(float, string.split(out[-1])[8:11])
# Save ENCLOSED, DIRECT, and their average estimates of fwhm for later use
psf2 = [psfs[0],psfs[2],(psfs[0]+psfs[2])/2]

iraf.imrename (stack,"sex.fits")

# Run source extractor to find objects in the frame
os.system('sex sex.fits -c default_is2.sex')
data = readsex('sex.cat')
xs = data[0]['X_IMAGE']
ys = data[0]['Y_IMAGE']
# Only keep the objects that have a zero flag
flags = data[0]['FLAGS']
gids = num.nonzero(num.equal(flags, 0))
xs = num.take(xs, gids)
ys = num.take(ys, gids)
f = open('objects.dat','w')
for i in range(len(xs)):  print >>f, xs[i],ys[i]
f.close()

iraf.imrename ("sex.fits",stack)

nuke('sex.cat');  nuke('sex.list')
iraf.tvmark (1, coords="objects.dat", mark="circle", radii="10",color=205)

# Get a new set of offsets from the source extracted objects
out = iraf.imcentroid ("@flist", reference=stack,coords="objects.dat",
                       maxshift=20, shifts="offsets.dat",verbose=0,Stdout=1)
out = map(string.split, out)
new_offsets = [[],[]]
for line in out:
   if len(line) == 0: continue
   if re.search('fits', line[0]):
      new_offsets[0].append(float(line[1]))
      new_offsets[1].append(float(line[3]))
new_offsets = num.array(new_offsets)
dif = offsets - new_offsets
ave = num.average(dif, axis=1)

# Newer-offsets is the new_offest, but shifted by the average difference
#  between the old and new.
newer_offsets = new_offsets + ave[:, num.NewAxis]
f = open('newer_offsets.dat', 'w')
for i in range(len(newer_offsets[0])):
   print >>f, newer_offsets[0][i], newer_offsets[1][i]
f.close()

stacka = stack.replace('.fits','a.fits')
stacka_s = stack.replace('.fits','a_sigma1.fits')
stackb = stack.replace('.fits','b.fits')
stackb_s = stack.replace('.fits','b_sigma1.fits')
nuke(stacka);  nuke(stackb); nuke(stacka_s);  nuke(stackb_s)

# Second and third attempt at imcombine.  a ==> newer offests.  b==> newer
#   offsets and sigma-clipping.
iraf.imcombine("@flist",stacka,offsets="newer_offsets.dat",combine="average", 
               masktyp="badvalue", maskval=1., scale="none", sigmas=stacka_s)

iraf.imcombine("@flist",stackb,offsets="newer_offsets.dat",combine="average", 
               masktyp="badvalue", maskval=1., scale="none",reject="pclip", 
               pclip=-0.5,lsigma=5.,hsigma=5.,nkeep=5, sigmas=stackb_s)

# Okay, let's try something a bit different.  Use geotran and give xshift
#   and yshift options as the offsets.  This allows for non-integer shifts
shift_list = []
# make a zero-image for padding:
nuke('zero.fits')
iraf.imarith(stackb, "*", 0.0, 'zero.fits')
for i in range(len(filelist)):
   nuke('temp.fits'); nuke('temp_mask.fits')
   iraf.imcopy('zero.fits','temp.fits')
   iraf.imcopy('zero.fits','temp_mask.fits')
   iraf.imarith('temp_mask.fits','+',10.0, 'temp_mask.fits')
   iraf.imcopy(filelist[i], 'temp.fits[1:1024,1:1024]')  
   shifted = filelist[i].replace('.fits','_shift.fits')
   shift_list.append(shifted)
   bpm = get_header(filelist[i], 'BPM', str)
   bpmf = bpm.replace('.pl','.fits')
   bpmf_shift = bpm.replace('.pl','_shift.fits')
   bpm_shift = bpm.replace('.pl','_shift.pl')

   nuke(shifted)
   nuke(bpmf); nuke(bpmf_shift); nuke(bpm_shift)
   # first, we convert the pixel mask into fits format
   iraf.imcopy(bpm, bpmf)
   iraf.imarith(bpmf, "*", 10.0, bpmf)
   iraf.imcopy(bpmf, 'temp_mask.fits[1:1024,1:1024]')
   iraf.geotran('temp.fits', shifted, database="", interpolant='linear', 
         boundary='nearest', fluxconserve=1, xshift=newer_offsets[0][i],
         yshift=newer_offsets[1][i])
   iraf.geotran('temp_mask.fits', bpmf_shift, database="", 
         interpolant='linear', boundary='nearest', fluxconserve=1, 
         xshift=newer_offsets[0][i], yshift=newer_offsets[1][i])
   # Now fix up the pixel mask
   iraf.imreplace(bpmf_shift, lower=0.02, upper='INDEF', value=1.0)
   iraf.imcopy(bpmf_shift, bpm_shift)
   iraf.hedit(shifted, 'BPM', bpm_shift, verify=0, update=1, add=1)

stacks = stack.replace('.fits','s.fits')
stacks_s = stack.replace('.fits','s_sigma1.fits')
nuke(stacks);  nuke(stacks_s)
# We simply imcombine with no offsets, as these have been done by geotran.
shift_list = string.join(shift_list, ',')
iraf.imcombine(shift_list,stacks,combine="average", 
               masktyp="badvalue", maskval=1., scale="none", sigmas=stacks_s)


iraf.display(stacka,1,fill=0,erase=0,border_erase=1,zscale=0,zrange=0,
             z1=-500,z2=500)
out = iraf.imexamine (stacka,1,imagecur="xy.in", Stdout=1)
psfs = map(float,string.split(out[-1])[8:11])
psfa2 = [psfs[0],psfs[2],(psfs[0]+psfs[2])/2]
# The difference in fwhm before and after recomputing offsets:
psf_dif = psfa2[2] - psf2[2]

if (psf_dif <= 0.):
   for i in range(len(filelist)):
      iraf.hedit(filelist[i], 'XOFF2', newer_offsets[0][i], verify=0, add=1)
      iraf.hedit(filelist[i], 'YOFF2', newer_offsets[1][i], verify=0, add=1)
   print "Headers updated"

else:
   print "" ; print "" 
   print " Headers not changed"
   print ""

nuke('comblist_'+sname)
f = open('comblist_'+sname, 'w')
for i in range(len(filelist)):
   file = filelist[i]
   rax = get_header(file, 'RA', float)
   decx = get_header(file, 'DEC', float)
   xs = get_header(file, 'XOFF2', float)
   ys = get_header(file, 'YOFF2', float)
 
   print >>f, "      ",file," ",rax," ",decx," ",xs," ",ys
