/*                     RW_FITS
*
*  Collection of routines for reading and writing FITS-standard files
*
*  types:
*     - fits:   struct with all information possibly needed from a fits
*               file

*/


#ifndef _RW_FITS
#define _RW_FITS
#endif

#ifndef _NETINET_IN_H
#include <netinet/in.h>
#endif

struct other_node {
   char key[9];              /* Key name */
   short int eq;             /* is there an = in column 9-10? */
   char value[71];           /* value or ASCII strings associated with key */
   struct other_node *next;  /* pointer to next in the linked list */
};

typedef struct other_node key_node;

typedef struct 
   {	   
      int simple;              
      int bitpix;              
      int n_axis;              
      int *naxis;            
      int blank;               
      int useblank;
      float *crpix;            
      float datamax;
      float datamin;           
      float *crval;          
      float *cdelt;          
		float *cd;
		float *pc;
      char object[80];
      key_node *other;
      float *data;             
   } fits;

extern int read_fits_file(char *filename, fits *);
extern int write_fits_file(char *filename, fits);
extern int read_fits(FILE *, fits *); 
extern int cp_header(fits , fits *);
extern int write_header(FILE *, fits );
extern int write_data(FILE *, fits );
extern int read_header(FILE *, fits *);
extern void print_header( fits );
extern int cp_data( fits, fits *);
extern float read_fits_pixel( FILE *, int);
extern int search_key(fits, char *, char *);
extern int change_key(fits, char *, char *);
extern int alloc_fits ( fits * );
extern int init_fits (fits *);
extern long int get_ndata(fits);
extern char *get_tag(fits *, char *);
extern fits *new_fits();
extern void free_fits_struct(fits *f);
